@@ -108,6 +108,7 @@ extern "C" {
 #define NCKEY_EXIT      preterunicode(133)
 #define NCKEY_PRINT     preterunicode(134)
 #define NCKEY_REFRESH   preterunicode(135)
+#define NCKEY_SEPARATOR preterunicode(136)
 // these keys aren't generally available outside of the kitty protocol
 #define NCKEY_CAPS_LOCK    preterunicode(150)
 #define NCKEY_SCROLL_LOCK  preterunicode(151)
@@ -143,6 +144,8 @@ extern "C" {
 #define NCKEY_RSUPER       preterunicode(180)
 #define NCKEY_RHYPER       preterunicode(181)
 #define NCKEY_RMETA        preterunicode(182)
+#define NCKEY_L3SHIFT      preterunicode(183)
+#define NCKEY_L5SHIFT      preterunicode(184)
 // mouse events. We encode which button was pressed into the char32_t,
 // but position information is embedded in the larger ncinput event.
 #define NCKEY_MOTION    preterunicode(200) // no buttons pressed
