@@ -6,7 +6,11 @@ extern "C" {
 #endif
 
 #include <time.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <stdint.h>
+#include <pthread.h>
 #include <sys/types.h>
 
 #define NANOSECS_IN_SEC 1000000000ul
@@ -16,6 +20,10 @@ extern "C" {
 #endif
 
 #ifdef  __MINGW64__
+static inline char
+path_separator(void){
+  return '\\';
+}
 #define NL "\r\n"
 #include <Lmcons.h>
 #include <winsock2.h>
@@ -38,13 +46,12 @@ extern "C" {
 #define SIGQUIT 0
 #define SIGCONT 0
 #define SIGWINCH 0
+#define SIGSTOP 0
 #define gettimeofday mingw_gettimeofday
 #define sigaddset(x, y)
 typedef struct siginfo_t {
   int aieeee;
 } siginfo_t;
-// not declared in MSYS2 header files, but implemented...?
-int faccessat(int dirfd, const char *pathname, int mode, int flags);
 #define sigset_t int
 #define nl_langinfo(x) "UTF-8"
 #define ppoll(w, x, y, z) WSAPoll((w), (x), (y))
@@ -57,6 +64,10 @@ struct winsize {
 };
 #define WNOHANG 0
 #else
+static inline char
+path_separator(void){
+  return '/';
+}
 #define NL "\n"
 #include <poll.h>
 #include <termios.h>
@@ -66,10 +77,16 @@ struct winsize {
 #include <sys/ioctl.h>
 #endif
 
-#ifndef _USE_GNU
-// FIXME actually implement this honoring c: CLOCK_MONOTONIC
-#define pthread_cond_clockwait(a, b, c, d) pthread_cond_timedwait(a, b, d)
-#endif
+// get the default data directory (heap-allocated). on unix, this is compiled
+// in from builddef.h. on windows, we try to look it up in the registry (it
+// ought have been written during installation), and fall back to builddef.h.
+char* notcurses_data_dir(void);
+
+// initializes a pthread_cond_t to use CLOCK_MONOTONIC (as opposed to the
+// default CLOCK_REALTIME) if possible. if not possible, initializes a
+// regular ol' CLOCK_REALTIME condvar. this eliminates the need for
+// pthread_cond_clockwait(), which is highly nonportable.
+int pthread_condmonotonic_init(pthread_cond_t* cond);
 
 int set_fd_nonblocking(int fd, unsigned state, unsigned* oldstate);
 int set_fd_cloexec(int fd, unsigned state, unsigned* oldstate);
@@ -86,6 +103,15 @@ ns_to_timespec(uint64_t ns, struct timespec* ts){
   return ts;
 }
 
+static inline uint64_t
+clock_getns(clockid_t clockid){
+  struct timespec tspec;
+  if(clock_gettime(clockid, &tspec) < 0){
+    return 0;
+  }
+  return timespec_to_ns(&tspec);
+}
+
 // compatibility wrappers for code available only on certain operating systems.
 // this file is not installed, but only consumed during compilation. if we're
 // on an operating system which implements a given function, it won't be built.
@@ -93,10 +119,37 @@ int clock_nanosleep(clockid_t clockid, int flags,
                     const struct timespec *request,
                     struct timespec *remain);
 
+__attribute__ ((nonnull (2))) __attribute__ ((malloc))
+static inline char*
+notcurses_data_path(const char* ddir, const char* f){
+  char* datadir = NULL;
+  if(ddir == NULL){
+    datadir = notcurses_data_dir();
+    if(datadir == NULL){
+      return NULL;
+    }
+    ddir = datadir;
+  }
+  const size_t dlen = strlen(ddir);
+  // cast is for benefit of c++ callers, sigh
+  char* path = (char*)malloc(dlen + 1 + strlen(f) + 1);
+  if(path == NULL){
+    free(datadir);
+    return NULL;
+  }
+  strcpy(path, ddir);
+  free(datadir);
+  path[dlen] = path_separator();
+  strcpy(path + dlen + 1, f);
+  return path;
+}
+
 #ifdef __cplusplus
 }
 #else
+#ifdef __MINGW64__
 char* strndup(const char* str, size_t size);
 #endif
+#endif
 
 #endif
