@@ -17,11 +17,11 @@
 static const int MIN_SUPPORTED_ROWS = 24;
 static const int MIN_SUPPORTED_COLS = 76; // allow a bit of margin, sigh
 
+static char *datadir;
 static int democount;
 static demoresult* results;
-static char *datadir = NOTCURSES_SHARE;
 
-static const char DEFAULT_DEMO[] = "ixetunchmdbkywgarvlsfjqzo";
+static const char DEFAULT_DEMO[] = "ixetunchmdbkywjgarvlsfqzo";
 
 atomic_bool interrupted = ATOMIC_VAR_INIT(false);
 // checked following demos, whether aborted, failed, or otherwise
@@ -43,21 +43,8 @@ const demoresult* demoresult_lookup(int idx){
   return &results[idx];
 }
 
-static inline const char*
-path_separator(void){
-#ifdef __MINGW64__
-  return "\\";
-#else
-  return "/";
-#endif
-}
-
 char* find_data(const char* datum){
-  char* path = malloc(strlen(datadir) + 1 + strlen(datum) + 1);
-  strcpy(path, datadir);
-  strcat(path, path_separator());
-  strcat(path, datum);
-  return path;
+  return notcurses_data_path(datadir, datum);
 }
 
 float delaymultiplier = 1;
@@ -71,8 +58,9 @@ struct timespec demodelay = {
 // the "jungle" demo has non-free material embedded into it, and is thus
 // entirely absent (can't just be disabled). supply a stub here.
 #ifdef DFSG_BUILD
-int jungle_demo(struct notcurses* nc){
+int jungle_demo(struct notcurses* nc, uint64_t startns){
   (void)nc;
+  (void)startns;
   return -1;
 }
 #endif
@@ -81,7 +69,7 @@ int jungle_demo(struct notcurses* nc){
 // be disabled) or a non-multimedia build (all images/videos must be disabled).
 static struct {
   const char* name;
-  int (*fxn)(struct notcurses*);
+  int (*fxn)(struct notcurses*, uint64_t startns);
   bool dfsg_disabled;             // disabled for DFSG builds
 } demos[26] = {
   { "animate", animate_demo, false, },
@@ -91,22 +79,22 @@ static struct {
   { "eagle", eagle_demo, true, },
   { "fission", fission_demo, false, },
   { "grid", grid_demo, false, },
-  { "highcon", highcontrast_demo, false, },
-  { "intro", intro, false, },
+  { "highcon", highcon_demo, false, },
+  { "intro", intro_demo, false, },
   { "jungle", jungle_demo, true, },
   { "keller", keller_demo, true, },
   { "luigi", luigi_demo, true, },
   { "mojibake", mojibake_demo, false, },
   { "normal", normal_demo, false, },
-  { "outro", outro, false, },
-  { NULL, NULL, false, }, // pango
+  { "outro", outro_demo, false, },
+  { NULL, NULL, false, }, // it's a secret to everyone
   { "qrcode", qrcode_demo, false, }, // is blank without USE_QRCODEGEN
   { "reel", reel_demo, false, },
-  { "sliders", sliding_puzzle_demo, false, },
+  { "sliders", sliders_demo, false, },
   { "trans", trans_demo, false, },
-  { "uniblock", unicodeblocks_demo, false, },
+  { "uniblock", uniblock_demo, false, },
   { "view", view_demo, true, },
-  { "whiteout", witherworm_demo, false, },
+  { "whiteout", whiteout_demo, false, },
   { "xray", xray_demo, false, },
   { "yield", yield_demo, false, },
   { "zoo", zoo_demo, false, },
@@ -131,7 +119,6 @@ usage_expo(FILE* out, struct ncdirect* n, const char* op, const char* expo){
   fprintf(out, "%s\n", expo);
 }
 
-// FIXME stylize this a little
 static void
 usage(const char* exe, int status){
   FILE* out = status == EXIT_SUCCESS ? stdout : stderr;
@@ -206,9 +193,7 @@ ext_demos(struct notcurses* nc, const char* spec){
   }
   memset(results, 0, sizeof(*results) * strlen(spec));
   democount = strlen(spec);
-  struct timespec start, now;
-  clock_gettime(CLOCK_MONOTONIC, &start);
-  uint64_t prevns = timespec_to_ns(&start);
+  uint64_t prevns = clock_getns(CLOCK_MONOTONIC);
   for(size_t i = 0 ; i < strlen(spec) ; ++i){
     results[i].selector = spec[i];
   }
@@ -229,11 +214,10 @@ ext_demos(struct notcurses* nc, const char* spec){
     uint64_t stdc = NCCHANNELS_INITIALIZER(0, 0, 0, 0, 0, 0);
     ncplane_set_base(n, "", 0, stdc);
 
-    hud_schedule(demos[idx].name);
-    ret = demos[idx].fxn(nc);
+    hud_schedule(demos[idx].name, prevns);
+    ret = demos[idx].fxn(nc, prevns);
     notcurses_stats_reset(nc, &results[i].stats);
-    clock_gettime(CLOCK_MONOTONIC, &now);
-    uint64_t nowns = timespec_to_ns(&now);
+    uint64_t nowns = clock_getns(CLOCK_MONOTONIC);
     results[i].timens = nowns - prevns;
     prevns = nowns;
     results[i].result = ret;
@@ -338,21 +322,29 @@ handle_opts(int argc, char** argv, notcurses_options* opts, FILE** json_output){
 }
 
 static int
-table_segment_color(struct ncplane* n, const char* str, const char* delim, unsigned color){
+table_segment_color(struct ncplane* n, const char* str, const char* delim,
+                    const char* ascdelim, unsigned color){
   ncplane_set_fg_rgb(n, color);
   if(ncplane_putstr(n, str) < 0){
     return -1;
   }
   ncplane_set_fg_rgb8(n, 178, 102, 255);
-  if(ncplane_putstr(n, delim) < 0){
-    return -1;
+  if(notcurses_canutf8(ncplane_notcurses(n))){
+    if(ncplane_putstr(n, delim) < 0){
+      return -1;
+    }
+  }else{
+    if(ncplane_putstr(n, ascdelim) < 0){
+      return -1;
+    }
   }
   return 0;
 }
 
 static int
-table_segment(struct ncplane* n, const char* str, const char* delim){
-  return table_segment_color(n, str, delim, 0xffffff);
+table_segment(struct ncplane* n, const char* str, const char* delim,
+              const char* ascdelim){
+  return table_segment_color(n, str, delim, ascdelim, 0xffffff);
 }
 
 static int
@@ -377,7 +369,7 @@ summary_json(FILE* f, const char* spec, int rows, int cols){
       continue;
     }
     ret |= (fprintf(f, "\"%s\":{\"bytes\":\"%"PRIu64"\",\"frames\":\"%"PRIu64"\",\"ns\":\"%"PRIu64"\"}%s",
-                    demos[results[i].selector - 'a'].name, results[i].stats.render_bytes,
+                    demos[results[i].selector - 'a'].name, results[i].stats.raster_bytes,
                     results[i].stats.renders, results[i].timens, i < strlen(spec) - 1 ? "," : "") < 0);
   }
   ret |= (fprintf(f, "}}}\n") < 0);
@@ -388,35 +380,44 @@ static int
 summary_table(struct notcurses* nc, const char* spec, bool canimage, bool canvideo){
   notcurses_leave_alternate_screen(nc);
   struct ncplane* n = notcurses_stdplane(nc);
+  ncplane_set_bg_default(n);
   ncplane_set_scrolling(n, true);
   bool failed = false;
   uint64_t totalbytes = 0;
   long unsigned totalframes = 0;
-  uint64_t totalrenderns = 0;
-  uint64_t totalwriteoutns = 0;
   ncplane_putchar(n, '\n');
-  table_segment(n, "             runtime", "│");
-  table_segment(n, " frames", "│");
-  table_segment(n, "output(B)", "│");
-  table_segment(n, "    FPS", "│");
-  table_segment(n, "%r", "│");
-  table_segment(n, "%a", "│");
-  table_segment(n, "%w", "│");
-  table_segment(n, "TheoFPS", "║\n══╤════════╤════════╪═══════╪═════════╪═══════╪══╪══╪══╪═══════╣\n");
-  char timebuf[PREFIXSTRLEN + 1];
-  char tfpsbuf[PREFIXSTRLEN + 1];
-  char totalbuf[BPREFIXSTRLEN + 1];
+  // FIXME this shouldn't be necessary, but without it, late in 2.4.x we
+  // stopped printing the table header. see #2389.
+  notcurses_render(nc);
+  const char* sep;
+  if(notcurses_canutf8(ncplane_notcurses(n))){
+    sep = u8"│";
+  }else{
+    sep = "|";
+  }
+  table_segment(n, "             runtime", "│", "|");
+  table_segment(n, " frames", "│", "|");
+  table_segment(n, "output(B)", "│", "|");
+  table_segment(n, "    FPS", "│", "|");
+  table_segment(n, "%r", "│", "|");
+  table_segment(n, "%a", "│", "|");
+  table_segment(n, "%w", "│", "|");
+  table_segment(n, "TheoFPS", "║\n══╤════════╤════════╪═══════╪═════════╪═══════╪══╪══╪══╪═══════╣\n",
+                "|\n--+--------+--------+-------+---------+-------+--+--+--+-------|\n");
+  char timebuf[NCPREFIXSTRLEN + 1];
+  char tfpsbuf[NCPREFIXSTRLEN + 1];
+  char totalbuf[NCBPREFIXSTRLEN + 1];
   uint64_t nsdelta = 0;
   for(size_t i = 0 ; i < strlen(spec) ; ++i){
     nsdelta += results[i].timens;
-    qprefix(results[i].timens, NANOSECS_IN_SEC, timebuf, 0);
-    bprefix(results[i].stats.render_bytes, 1, totalbuf, 0);
+    ncqprefix(results[i].timens, NANOSECS_IN_SEC, timebuf, 0);
+    ncbprefix(results[i].stats.raster_bytes, 1, totalbuf, 0);
     uint64_t divisor = results[i].stats.render_ns + results[i].stats.writeout_ns + results[i].stats.raster_ns;
     if(divisor){
-      qprefix((uintmax_t)results[i].stats.writeouts * NANOSECS_IN_SEC * 1000 / divisor,
+      ncqprefix((uintmax_t)results[i].stats.writeouts * NANOSECS_IN_SEC * 1000 / divisor,
               1000, tfpsbuf, 0);
     }else{
-      qprefix(0, NANOSECS_IN_SEC, tfpsbuf, 0);
+      ncqprefix(0, NANOSECS_IN_SEC, tfpsbuf, 0);
     }
     uint32_t rescolor;
     if(results[i].result < 0){
@@ -431,21 +432,24 @@ summary_table(struct notcurses* nc, const char* spec, bool canimage, bool canvid
     ncplane_set_fg_rgb(n, rescolor);
     ncplane_printf(n, "%2llu", (unsigned long long)(i + 1)); // windows has %zu problems
     ncplane_set_fg_rgb8(n, 178, 102, 255);
-    ncplane_putwc(n, L'│');
+    ncplane_putegc(n, sep, NULL);
     ncplane_set_fg_rgb(n, rescolor);
     ncplane_printf(n, "%8s", demos[results[i].selector - 'a'].name);
     ncplane_set_fg_rgb8(n, 178, 102, 255);
-    ncplane_printf(n, "│%*ss│%7" PRIu64 "│%*s│%7.1f│%2" PRId64 "│%2" PRId64 "│%2" PRId64 "│%*s║",
-           PREFIXFMT(timebuf), results[i].stats.renders, BPREFIXFMT(totalbuf),
+    ncplane_printf(n, "%s%*ss%s%7" PRIu64 "%s%*s%s%7.1f%s%2" PRId64 "%s%2" PRId64 "%s%2" PRId64 "%s%*s%s",
+           sep, NCPREFIXFMT(timebuf), sep,
+           results[i].stats.renders, sep,
+           NCBPREFIXFMT(totalbuf), sep,
            results[i].timens ?
-            results[i].stats.renders / ((double)results[i].timens / NANOSECS_IN_SEC) : 0.0,
+            results[i].stats.renders / ((double)results[i].timens / NANOSECS_IN_SEC) : 0.0, sep,
            (results[i].timens ?
-            results[i].stats.render_ns * 100 / results[i].timens : 0),
+            results[i].stats.render_ns * 100 / results[i].timens : 0), sep,
            (results[i].timens ?
-            results[i].stats.raster_ns * 100 / results[i].timens : 0),
+            results[i].stats.raster_ns * 100 / results[i].timens : 0), sep,
            (results[i].timens ?
-            results[i].stats.writeout_ns * 100 / results[i].timens : 0),
-           PREFIXFMT(tfpsbuf));
+            results[i].stats.writeout_ns * 100 / results[i].timens : 0), sep,
+           NCPREFIXFMT(tfpsbuf),
+           notcurses_canutf8(ncplane_notcurses(n)) ? "║" : "|");
     ncplane_set_fg_rgb(n, rescolor);
     ncplane_printf(n, "%s\n", results[i].result < 0 ? "FAILED" :
                    results[i].result > 0 ? "ABORTED" :
@@ -454,18 +458,17 @@ summary_table(struct notcurses* nc, const char* spec, bool canimage, bool canvid
       failed = true;
     }
     totalframes += results[i].stats.renders;
-    totalbytes += results[i].stats.render_bytes;
-    totalrenderns += results[i].stats.render_ns;
-    totalwriteoutns += results[i].stats.writeout_ns;
+    totalbytes += results[i].stats.raster_bytes;
   }
-  qprefix(nsdelta, NANOSECS_IN_SEC, timebuf, 0);
-  bprefix(totalbytes, 1, totalbuf, 0);
-  table_segment(n, "", "══╧════════╧════════╪═══════╪═════════╪═══════╧══╧══╧══╧═══════╝\n");
+  ncqprefix(nsdelta, NANOSECS_IN_SEC, timebuf, 0);
+  ncbprefix(totalbytes, 1, totalbuf, 0);
+  table_segment(n, "", "══╧════════╧════════╪═══════╪═════════╪═══════╧══╧══╧══╧═══════╝\n",
+                "--+--------+--------+-------+---------+-------+--+--+--+-------+\n");
   ncplane_putstr(n, "            ");
-  table_printf(n, "│", "%*ss", PREFIXFMT(timebuf));
-  table_printf(n, "│", "%7lu", totalframes);
-  table_printf(n, "│", "%*s", BPREFIXFMT(totalbuf));
-  //table_printf(nc, "│", "%7.1f", nsdelta ? totalframes / ((double)nsdelta / NANOSECS_IN_SEC) : 0);
+  table_printf(n, sep, "%*ss", NCPREFIXFMT(timebuf));
+  table_printf(n, sep, "%7lu", totalframes);
+  table_printf(n, sep, "%*s", NCBPREFIXFMT(totalbuf));
+  //table_printf(nc, sep, "%7.1f", nsdelta ? totalframes / ((double)nsdelta / NANOSECS_IN_SEC) : 0);
   ncplane_putchar(n, '\n');
   ncplane_set_fg_rgb8(n, 0xfe, 0x20, 0x76); // PANTONE Strong Red C + 3x0x20
 #ifdef DFSG_BUILD
@@ -527,10 +530,10 @@ int main(int argc, char** argv){
   if((nc = notcurses_init(&nopts, NULL)) == NULL){
     return EXIT_FAILURE;
   }
-  notcurses_mouse_enable(nc);
+  notcurses_mice_enable(nc, NCMICE_BUTTON_EVENT | NCMICE_DRAG_EVENT);
   const bool canimage = notcurses_canopen_images(nc);
   const bool canvideo = notcurses_canopen_videos(nc);
-  int dimx, dimy;
+  unsigned dimx, dimy;
   ncplane_dim_yx(notcurses_stdplane(nc), &dimy, &dimx);
   if(input_dispatcher(nc)){
     goto err;
