@@ -49,6 +49,7 @@ static struct ncplane* debug; // "debug info" modal popup
 
 #define MENUSTR_TOGGLE_HUD "Toggle HUD"
 #define MENUSTR_TOGGLE_PLOT "Toggle FPS plot"
+#define MENUSTR_REDRAW_SCREEN "Redraw the screen"
 #define MENUSTR_RESTART "Restart"
 #define MENUSTR_ABOUT "About"
 #define MENUSTR_DEBUG "Debug info"
@@ -86,7 +87,7 @@ debug_toggle(struct notcurses* nc){
     debug = NULL;
     return;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   fbuf f;
   if(fbuf_init_small(&f)){
@@ -123,8 +124,8 @@ debug_toggle(struct notcurses* nc){
     return;
   }
   fbuf_free(&f);
-  for(int y = 0 ; y < ncplane_dim_y(n) ; ++y){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+  for(unsigned y = 0 ; y < ncplane_dim_y(n) ; ++y){
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT);
     nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT);
     ncplane_putc_yx(n, y, ncplane_dim_x(n) - 1, &c);
@@ -144,7 +145,7 @@ about_toggle(struct notcurses* nc){
   }
   const int ABOUT_ROWS = 9;
   const int ABOUT_COLS = 40;
-  int dimy;
+  unsigned dimy;
   notcurses_term_dim_yx(nc, &dimy, NULL);
   ncplane_options nopts = {
     .y = 3,
@@ -171,9 +172,9 @@ about_toggle(struct notcurses* nc){
     ncplane_printf_aligned(n, 4, NCALIGN_RIGHT, "restart Ctrl+R  ");
     ncplane_printf_aligned(n, 5, NCALIGN_CENTER, "q quit");
     ncplane_putstr_aligned(n, 7, NCALIGN_CENTER, "\u00a9 nick black <nickblack@linux.com>");
-    nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-    nccell lr = CELL_TRIVIAL_INITIALIZER, ll = CELL_TRIVIAL_INITIALIZER;
-    nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+    nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+    nccell lr = NCCELL_TRIVIAL_INITIALIZER, ll = NCCELL_TRIVIAL_INITIALIZER;
+    nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
     channels = 0;
     ncchannels_set_fg_rgb(&channels, 0xc020c0);
     ncchannels_set_bg_rgb(&channels, 0);
@@ -270,6 +271,13 @@ bool menu_or_hud_key(struct notcurses *nc, const struct ncinput *ni){
     debug_toggle(nc);
     return true;
   }
+  if(tmpni.id == 'L' && !tmpni.alt && tmpni.ctrl){
+    if(menu){
+      ncmenu_rollup(menu);
+    }
+    notcurses_refresh(nc, NULL, NULL);
+    return true;
+  }
   if(tmpni.id == 'R' && !tmpni.alt && tmpni.ctrl){
     if(menu){
       ncmenu_rollup(menu);
@@ -297,6 +305,7 @@ struct ncmenu* menu_create(struct notcurses* nc){
   struct ncmenu_item demo_items[] = {
     { .desc = MENUSTR_TOGGLE_HUD, .shortcut = { .id = 'H', }, },
     { .desc = MENUSTR_TOGGLE_PLOT, .shortcut = { .id = 'P', }, },
+    { .desc = MENUSTR_REDRAW_SCREEN, .shortcut = { .id = 'L', .ctrl = true }, },
     { .desc = NULL, },
     { .desc = MENUSTR_RESTART, .shortcut = { .id = 'R', .ctrl = true, }, },
     { .desc = MENUSTR_QUIT, .shortcut = { .id = 'q', }, },
@@ -337,9 +346,9 @@ struct ncmenu* menu_create(struct notcurses* nc){
 static int
 hud_refresh(struct ncplane* n){
   ncplane_erase(n);
-  nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-  nccell lr = CELL_TRIVIAL_INITIALIZER, ll = CELL_TRIVIAL_INITIALIZER;
-  nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+  nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+  nccell lr = NCCELL_TRIVIAL_INITIALIZER, ll = NCCELL_TRIVIAL_INITIALIZER;
+  nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
   if(nccells_rounded_box(n, NCSTYLE_NONE, 0, &ul, &ur, &ll, &lr, &hl, &vl)){
     return -1;
   }
@@ -382,7 +391,7 @@ hud_print_finished(elem* list){
       break;
     }
     if(hud){
-      nccell c = CELL_TRIVIAL_INITIALIZER;
+      nccell c = NCCELL_TRIVIAL_INITIALIZER;
       ncplane_base(hud, &c);
       ncplane_set_bg_rgb(hud, nccell_bg_rgb(&c));
       ncplane_set_bg_alpha(hud, NCALPHA_BLEND);
@@ -392,9 +401,13 @@ hud_print_finished(elem* list){
       if(ncplane_printf_yx(hud, line, 1, "%d", e->frames) < 0){
         return -1;
       }
-      if(ncplane_printf_yx(hud, line, 7, "%"PRIu64".%03"PRIu64"s",
-                           e->totalns / NANOSECS_IN_SEC,
-                           (e->totalns % NANOSECS_IN_SEC) / 1000000) < 0){
+      char buf[NCPREFIXCOLUMNS + 2];
+      ncnmetric(e->totalns, sizeof(buf), NANOSECS_IN_SEC, buf, 0, 1000, '\0');
+      for(int x = 6 ; x < 14 - ncstrwidth(buf, NULL, NULL) ; ++x){
+        nccell ci = NCCELL_TRIVIAL_INITIALIZER;
+        ncplane_putc_yx(hud, 1, x, &ci);
+      }
+      if(ncplane_printf_yx(hud, line, 14 - ncstrwidth(buf, NULL, NULL), "%ss", buf) < 0){
         return -1;
       }
       if(ncplane_putstr_yx(hud, line, 16, e->name) < 0){
@@ -410,7 +423,7 @@ struct ncplane* hud_create(struct notcurses* nc){
   if(hud){
     return NULL;
   }
-  int dimx, dimy;
+  unsigned dimx, dimy;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   struct ncplane_options nopts = {
     // FPS graph is 6 rows tall; we want one row above it
@@ -420,7 +433,7 @@ struct ncplane* hud_create(struct notcurses* nc){
     .cols = HUD_COLS,
     .userptr = NULL,
     .name = "hud",
-    .resizecb = NULL,
+    .resizecb = ncplane_resize_placewithin,
     .flags = NCPLANE_OPTION_HORALIGNED |
              NCPLANE_OPTION_FIXED,
   };
@@ -508,8 +521,8 @@ int hud_completion_notify(const demoresult* result){
   return 0;
 }
 
-// inform the HUD of an upcoming demo
-int hud_schedule(const char* demoname){
+// inform the HUD of an upcoming demo, starting at startns.
+int hud_schedule(const char* demoname, uint64_t startns){
   elem* cure = malloc(sizeof(*cure));
   if(!cure){
     return -1;
@@ -518,10 +531,8 @@ int hud_schedule(const char* demoname){
   cure->name = strdup(demoname);
   cure->totalns = 0;
   cure->frames = 0;
+  cure->startns = startns;
   elems = cure;
-  struct timespec cur;
-  clock_gettime(CLOCK_MONOTONIC, &cur);
-  cure->startns = timespec_to_ns(&cur);
   return hud_print_finished(elems);
 }
 
@@ -597,7 +608,7 @@ int demo_render(struct notcurses* nc){
     }
     uint64_t ns = timespec_to_ns(&ts) - elems->startns;
     ++elems->frames;
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_base(hud, &c);
     ncplane_set_bg_rgb(hud, nccell_bg_rgb(&c));
     ncplane_set_bg_alpha(hud, NCALPHA_BLEND);
@@ -608,8 +619,14 @@ int demo_render(struct notcurses* nc){
     if(ncplane_printf_yx(hud, 1, 1, "%d", elems->frames) < 0){
       return -1;
     }
-    if(ncplane_printf_yx(hud, 1, 7, "%"PRIu64".%03"PRIu64"s", ns / NANOSECS_IN_SEC,
-                         (ns % NANOSECS_IN_SEC) / 1000000) < 0){
+    char buf[NCPREFIXCOLUMNS + 2];
+    ncnmetric(ns, sizeof(buf), NANOSECS_IN_SEC, buf, 0, 1000, '\0');
+    for(int x = 6 ; x < 14 - ncstrwidth(buf, NULL, NULL) ; ++x){
+      nccell ci = NCCELL_TRIVIAL_INITIALIZER;
+      ncplane_putc_yx(hud, 1, x, &ci);
+    }
+//fprintf(stderr, "[%s] %zu %d\n", buf, strlen(buf), ncstrwidth(buf, NULL, NULL));
+    if(ncplane_printf_yx(hud, 1, 14 - ncstrwidth(buf, NULL, NULL), "%ss", buf) < 0){
       return -1;
     }
     if(ncplane_putstr_yx(hud, 1, 16, elems->name) < 0){
@@ -632,7 +649,7 @@ int demo_render(struct notcurses* nc){
 
 int fpsgraph_init(struct notcurses* nc){
   const int PLOTHEIGHT = 6;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* stdn = notcurses_stddim_yx(nc, &dimy, &dimx);
   ncplane_options nopts = {
     .y = NCALIGN_BOTTOM,
