@@ -4,14 +4,18 @@
 #include <string.h>
 #include <unistd.h>
 #include <locale.h>
+#include <assert.h>
+#include <strings.h>
 #include <pthread.h>
 #include <semaphore.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #if defined(__linux__) || defined(__gnu_hurd__)
+#include <langinfo.h>
 #include <sys/utsname.h>
 #include <sys/sysinfo.h>
 #elif !defined(__MINGW64__)
+#include <langinfo.h>
 #include <sys/sysctl.h>
 #include <sys/utsname.h>
 #else
@@ -19,8 +23,28 @@
 #endif
 #include <notcurses/notcurses.h>
 #include "compat/compat.h"
+#include "builddef.h"
 #include "ncart.h"
 
+static inline char*
+find_data(const char* datum){
+  char* datadir = notcurses_data_dir();
+  if(datadir == NULL){
+    return NULL;
+  }
+  const size_t dlen = strlen(datadir);
+  char* path = malloc(dlen + 1 + strlen(datum) + 1);
+  if(path == NULL){
+    free(datadir);
+    return NULL;
+  }
+  strcpy(path, datadir);
+  path[dlen] = path_separator();
+  strcpy(path + dlen + 1, datum);
+  free(datadir);
+  return path;
+}
+
 typedef struct distro_info {
   const char* name;            // must match 'lsb_release -i'
   const char* logofile;        // kept at original aspect ratio, lain atop bg
@@ -38,7 +62,6 @@ typedef struct fetched_info {
   const char* shell;           // getenv("SHELL")
   char* term;                  // notcurses_detected_terminal(), heap-alloced
   char* lang;                  // getenv("LANG")
-  int dimy, dimx;              // extracted from xrandr
   char* cpu_model;             // FIXME don't handle hetero setups yet
   int core_count;
   // if there is no other logo found, fall back to a logo filched from neofetch
@@ -58,35 +81,40 @@ free_fetched_info(fetched_info* fi){
 
 static int
 fetch_env_vars(struct notcurses* nc, fetched_info* fi){
+#if defined(__APPLE__)
+  fi->desktop = "Aqua";
+#elif defined(__MINGW64__)
+  fi->desktop = "Metro";
+#else
   fi->desktop = getenv("XDG_CURRENT_DESKTOP");
+#endif
   fi->shell = getenv("SHELL");
   fi->term = notcurses_detected_terminal(nc);
   fi->lang = getenv("LANG");
+  if(fi->lang == NULL){
+    fi->lang = nl_langinfo(CODESET);
+  }
   return 0;
 }
 
-static distro_info distros[] = {
-  {
-    .name = "arch",
-    // from core/filesystem
-    .logofile = "/usr/share/pixmaps/archlinux.png",
-  }, {
-    .name = "artix",
-    // from system/filesystem
-    .logofile = "/usr/share/pixmaps/artixlinux-logo.png",
-  }, {
-    .name = "debian",
-    // from desktop-base package
-    .logofile = "/usr/share/desktop-base/debian-logos/logo-text-256.png",
-  }, {
-    .name = "fedora",
-    // from redhat-lsb-core package
-    .logofile = "/usr/share/pixmaps/fedora-logo.png",
-  }, {
-    .name = NULL,
-    .logofile = NULL,
-  },
-};
+// if nothing else is available, use a generic architecture based on compile
+static const char*
+fallback_cpuinfo(void){
+#if defined(__amd64__) || defined(_M_AMD64)
+  return "amd64";
+#elif defined(__aarch64__)
+  return "aarch64";
+#elif defined(__arm__)
+  return "arm";
+#elif defined(__i386__) || defined(_M_IX86)
+  return "i386";
+#elif defined(__ia64__)
+  return "ia64";
+#else
+  #warning "unknown target architecture"
+  return "unknown";
+#endif
+}
 
 static int
 fetch_bsd_cpuinfo(fetched_info* fi){
@@ -110,6 +138,34 @@ fetch_bsd_cpuinfo(fetched_info* fi){
   return 0;
 }
 
+static int
+fetch_windows_cpuinfo(fetched_info* fi){
+#ifdef __MINGW64__
+  SYSTEM_INFO info = {};
+  GetSystemInfo(&info);
+  switch(info.wProcessorArchitecture){
+    case PROCESSOR_ARCHITECTURE_AMD64:
+      fi->cpu_model = strdup("amd64"); break;
+    case PROCESSOR_ARCHITECTURE_ARM:
+      fi->cpu_model = strdup("ARM"); break;
+    case PROCESSOR_ARCHITECTURE_ARM64:
+      fi->cpu_model = strdup("AArch64"); break;
+    case PROCESSOR_ARCHITECTURE_IA64:
+      fi->cpu_model = strdup("Itanium"); break;
+    case PROCESSOR_ARCHITECTURE_INTEL:
+      fi->cpu_model = strdup("i386"); break;
+    default:
+      fi->cpu_model = strdup("Unknown processor"); break;
+  }
+  fi->core_count = info.dwNumberOfProcessors;
+#else
+  (void)fi;
+#endif
+  return 0;
+}
+
+// guess what? the form of /proc/cpuinfo is arch-dependent!
+// that's right, fuck you!
 static int
 fetch_cpu_info(fetched_info* fi){
   FILE* cpuinfo = fopen("/proc/cpuinfo", "re");
@@ -119,7 +175,8 @@ fetch_cpu_info(fetched_info* fi){
   }
   char buf[BUFSIZ];
   while(fgets(buf, sizeof(buf), cpuinfo)){
-#define CORE "core id"
+// works for both amd64 and ARM
+#define CORE "processor"
 // model name doesn't appear on all architectures, so fall back to vendor_id
 #define TAG "model name"
 #define VEND "vendor_id"
@@ -144,7 +201,8 @@ fetch_cpu_info(fetched_info* fi){
           fi->cpu_model = strdup(start);
         }
       }
-    }else if(strncmp(buf, CORE, strlen(CORE)) == 0){
+      // need strncasecmp() because ARM cpuinfo uses "Processor" ugh
+    }else if(strncasecmp(buf, CORE, strlen(CORE)) == 0){
       ++fi->core_count;
     }
 #undef VEND
@@ -154,49 +212,7 @@ fetch_cpu_info(fetched_info* fi){
   return 0;
 }
 
-static char*
-pipe_getline(const char* cmdline){
-  FILE* p = popen(cmdline, "re");
-  if(p == NULL){
-    fprintf(stderr, "Error running %s (%s)\n", cmdline, strerror(errno));
-    return NULL;
-  }
-  char* buf = malloc(BUFSIZ); // gatesv("BUFSIZ bytes is enough for anyone")
-  if(fgets(buf, BUFSIZ, p) == NULL){
-//fprintf(stderr, "Error reading from %s (%s)\n", cmdline, strerror(errno));
-    pclose(p);
-    free(buf);
-    return NULL;
-  }
-  // FIXME read any remaining junk so as to stave off SIGPIPEs?
-  if(pclose(p)){
-    fprintf(stderr, "Error closing pipe (%s)\n", strerror(errno));
-    free(buf);
-    return NULL;
-  }
-  return buf;
-}
-
-static int
-fetch_x_props(fetched_info* fi){
-  char* xrandr = pipe_getline("xrandr --current 2>/dev/null");
-  if(xrandr == NULL){
-    return -1;
-  }
-  char* randrcurrent = strstr(xrandr, " current ");
-  if(randrcurrent == NULL){
-    free(xrandr);
-    return -1;
-  }
-  randrcurrent += strlen(" current ");
-  if(sscanf(randrcurrent, "%d x %d", &fi->dimx, &fi->dimy) != 2){
-    free(xrandr);
-    return -1;
-  }
-  free(xrandr);
-  return 0;
-}
-
+#ifdef __linux__
 // Given a filename, check for its existence in the directories specified by
 // https://specifications.freedesktop.org/icon-theme-spec/latest/ar01s03.html.
 // Returns NULL if no such file can be found. Return value is heap-allocated.
@@ -217,10 +233,35 @@ get_xdg_logo(const char *spec){
   strcat(p, spec);
   return p;
 }
+#endif
 
 // FIXME deal more forgivingly with quotation marks
 static const distro_info*
 linux_ncneofetch(fetched_info* fi){
+  const distro_info* dinfo = NULL;
+#ifdef __linux__
+  static const distro_info distros[] = {
+    {
+      .name = "arch",
+      // from core/filesystem
+      .logofile = "/usr/share/pixmaps/archlinux-logo.png",
+    }, {
+      .name = "artix",
+      // from system/filesystem
+      .logofile = "/usr/share/pixmaps/artixlinux-logo.png",
+    }, {
+      .name = "debian",
+      // from desktop-base package
+      .logofile = "/usr/share/desktop-base/debian-logos/logo-text-256.png",
+    }, {
+      .name = "fedora",
+      // from redhat-lsb-core package
+      .logofile = "/usr/share/pixmaps/fedora-logo.png",
+    }, {
+      .name = NULL,
+      .logofile = NULL,
+    },
+  };
   FILE* osinfo = fopen("/etc/os-release", "re");
   if(osinfo == NULL){
     return NULL;
@@ -266,7 +307,6 @@ linux_ncneofetch(fetched_info* fi){
   if(distro == NULL){
     return NULL;
   }
-  const distro_info* dinfo = NULL;
   for(dinfo = distros ; dinfo->name ; ++dinfo){
     if(strcmp(dinfo->name, distro) == 0){
       break;
@@ -276,6 +316,9 @@ linux_ncneofetch(fetched_info* fi){
     fi->neologo = get_neofetch_art(distro);
   }
   free(distro);
+#else
+  (void)fi;
+#endif
   return dinfo;
 }
 
@@ -309,29 +352,81 @@ get_kernel(fetched_info* fi){
   }
   fprintf(stderr, "Unknown operating system via uname: %s\n", uts.sysname);
 #else
-  (void)fi;
+  OSVERSIONINFOEX osvi;
+  ZeroMemory(&osvi, sizeof(osvi));
+  osvi.dwOSVersionInfoSize = sizeof(osvi);
+  GetVersionExA((LPOSVERSIONINFOA)&osvi);
+  char ver[20]; // sure why not
+  snprintf(ver, sizeof(ver), "%lu.%lu", osvi.dwMajorVersion, osvi.dwMinorVersion);
+  fi->kernver = strdup(ver);
+  fi->kernel = strdup("WNT");
+  DWORD ptype;
+  if(GetProductInfo(osvi.dwMajorVersion,
+                    osvi.dwMinorVersion,
+                    osvi.wServicePackMajor,
+                    osvi.wServicePackMinor,
+                    &ptype)){
+    switch(ptype){
+      case PRODUCT_BUSINESS: fi->distro_pretty = strdup("Business"); break;
+      case PRODUCT_BUSINESS_N: fi->distro_pretty = strdup("Business N"); break;
+      case PRODUCT_CLUSTER_SERVER: fi->distro_pretty = strdup("HPC Edition"); break;
+      case PRODUCT_CLUSTER_SERVER_V: fi->distro_pretty = strdup("Server Hyper Core V"); break;
+      case PRODUCT_CORE: fi->distro_pretty = strdup("Windows 10 Home"); break;
+      case PRODUCT_CORE_COUNTRYSPECIFIC: fi->distro_pretty = strdup("Windows 10 Home China"); break;
+      case PRODUCT_CORE_N: fi->distro_pretty = strdup("Windows 10 Home N"); break;
+      case PRODUCT_CORE_SINGLELANGUAGE: fi->distro_pretty = strdup("Windows 10 Home Single Language"); break;
+      case PRODUCT_EDUCATION: fi->distro_pretty = strdup("Windows 10 Education"); break;
+      case PRODUCT_EDUCATION_N: fi->distro_pretty = strdup("Windows 10 Education N"); break;
+      case PRODUCT_ENTERPRISE: fi->distro_pretty = strdup("Windows 10 Enterprise"); break;
+      case PRODUCT_ENTERPRISE_EVALUATION: fi->distro_pretty = strdup("Windows 10 Enterprise Eval"); break;
+      case PRODUCT_ENTERPRISE_E: fi->distro_pretty = strdup("Windows 10 Enterprise E"); break;
+      case PRODUCT_ENTERPRISE_N: fi->distro_pretty = strdup("Windows 10 Enterprise N"); break;
+      case PRODUCT_ENTERPRISE_N_EVALUATION: fi->distro_pretty = strdup("Windows 10 Enterprise N Eval"); break;
+      case PRODUCT_ENTERPRISE_S: fi->distro_pretty = strdup("Windows 10 Enterprise 2015 LTSB"); break;
+      case PRODUCT_ENTERPRISE_S_EVALUATION: fi->distro_pretty = strdup("Windows 10 Enterprise 2015 LTSB Eval"); break;
+      case PRODUCT_ENTERPRISE_S_N: fi->distro_pretty = strdup("Windows 10 Enterprise 2015 LTSB N"); break;
+      case PRODUCT_ENTERPRISE_S_N_EVALUATION: fi->distro_pretty = strdup("Windows 10 Enterprise 2015 LTSB N Eval"); break;
+      case PRODUCT_HOME_BASIC: fi->distro_pretty = strdup("Home Basic"); break;
+      case PRODUCT_HOME_BASIC_N: fi->distro_pretty = strdup("Home Basic N"); break;
+      case PRODUCT_HOME_PREMIUM: fi->distro_pretty = strdup("Home Premium"); break;
+      case PRODUCT_HOME_PREMIUM_N: fi->distro_pretty = strdup("Home Premium N"); break;
+      case PRODUCT_HOME_PREMIUM_SERVER: fi->distro_pretty = strdup("Windows Home Server 2011"); break;
+      case PRODUCT_HOME_SERVER: fi->distro_pretty = strdup("Windows Storage Server 2008 R2 Essentials"); break;
+      case PRODUCT_HYPERV: fi->distro_pretty = strdup("Windows Hyper-V Server"); break;
+      case PRODUCT_IOTUAP: fi->distro_pretty = strdup("Windows 10 IoT Core"); break;
+      /*case PRODUCT_IOTUAPCOMMERCIAL: fi->distro_pretty = strdup("Windows 10 IoT Core Commercial"); break;
+      case PRODUCT_PRO_WORKSTATION: fi->distro_pretty = strdup("Windows 10 Pro for Workstations"); break;
+      case PRODUCT_PRO_WORKSTATION_N: fi->distro_pretty = strdup("Windows 10 Pro for Workstations N"); break;*/
+      case PRODUCT_PROFESSIONAL: fi->distro_pretty = strdup("Windows 10 Pro"); break;
+      case PRODUCT_PROFESSIONAL_N: fi->distro_pretty = strdup("Windows 10 Pro N"); break;
+      case PRODUCT_PROFESSIONAL_WMC: fi->distro_pretty = strdup("Professional with Media Center"); break;
+      case PRODUCT_ULTIMATE: fi->distro_pretty = strdup("Ultimate"); break;
+      case PRODUCT_ULTIMATE_N: fi->distro_pretty = strdup("Ultimate N"); break;
+      default: fi->distro_pretty = strdup("Unknown product"); break;
+    }
+  }
   return NCNEO_WINDOWS;
 #endif
   return NCNEO_UNKNOWN;
 }
 
+// windows distro_pretty gets handled in get_kernel() above
 static const distro_info*
 windows_ncneofetch(fetched_info* fi){
-  static const distro_info mswin = {
+  static distro_info mswin = {
     .name = "Windows",
-    .logofile = NULL, // FIXME
   };
+  mswin.logofile = find_data("Windows10Logo.png"),
   fi->neologo = get_neofetch_art("Windows");
-  fi->distro_pretty = NULL;
   return &mswin;
 }
 
 static const distro_info*
 freebsd_ncneofetch(fetched_info* fi){
-  static const distro_info fbsd = {
+  static distro_info fbsd = {
     .name = "FreeBSD",
-    .logofile = NULL, // FIXME
   };
+  fbsd.logofile = find_data("freebsd.png"),
   fi->neologo = get_neofetch_art("BSD"); // use big daemon logo
   fi->distro_pretty = NULL;
   return &fbsd;
@@ -361,38 +456,63 @@ xnu_ncneofetch(fetched_info* fi){
 
 static int
 drawpalette(struct notcurses* nc){
+  int showpl = 64; // show this many per line
   int psize = notcurses_palette_size(nc);
-  if(psize > 256){
-    psize = 256;
+  if(psize > NCPALETTESIZE){
+    psize = NCPALETTESIZE;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   if(dimx < 64){
     return -1;
   }
-  for(int y = 0 ; y < (psize + 63) / 64 ; ++y){
-    // we show a maximum of 64 palette entries per line
-    int toshow = psize - y * 64;
-    if(toshow > 64){
-      toshow = 64;
+  int scale = notcurses_canutf8(nc) ? 2 : 1; // use half blocks for 2*showpl
+  ncplane_cursor_move_yx(n, -1, 0);
+  for(int y = 0 ; y < (psize + (showpl * scale) - 1) / showpl / scale ; ++y){
+    // we show a maximum of showpl * scale palette entries per line
+    int toshow = psize - y * showpl;
+    if(toshow > showpl){
+      toshow = showpl;
     }
-    // center based on the number being shown on this line
-    if(ncplane_cursor_move_yx(n, -1, (dimx - toshow) / 2)){
+    if(ncplane_cursor_move_yx(n, -1, 0)){
       return -1;
     }
-    for(int x = (dimx - 64) / 2 ; x < dimx / 2 + 32 ; ++x){
-      const int truex = x - (dimx - 64) / 2;
-      if(y * 64 + truex >= psize){
-        break;
-      }
-      if(ncplane_set_bg_palindex(n, y * 64 + truex)){
-        return -1;
-      }
-      if(ncplane_putchar(n, ' ') == EOF){
+    ncplane_set_fg_default(n);
+    ncplane_set_bg_default(n);
+    for(unsigned x = 0 ; x < (dimx - toshow) / 2 ; ++x){
+      if(ncplane_putchar(n, ' ') < 0){
         return -1;
       }
     }
+    // center based on the number being shown on this line
+    for(unsigned x = (dimx - toshow) / 2 ; x < dimx / 2 + 32 ; ++x){
+      const int truex = x - (dimx - toshow) / 2;
+      if(y * showpl * scale + truex >= psize){
+        break;
+      }
+      if(ncplane_set_bg_palindex(n, y * showpl * scale + truex)){
+        return -1;
+      }
+      if(scale == 2){
+        if(ncplane_set_fg_palindex(n, (y * showpl * scale + truex + showpl) % psize)){
+          return -1;
+        }
+        if(ncplane_putegc(n, u8"▄", NULL) == EOF){
+          return -1;
+        }
+      }else{
+        if(ncplane_putchar(n, ' ') == EOF){
+          return -1;
+        }
+      }
+    }
+    ncplane_set_fg_default(n);
     ncplane_set_bg_default(n);
+    for(unsigned x = dimx / 2 + 32 ; x < dimx - 1 ; ++x){
+      if(ncplane_putchar(n, ' ') < 0){
+        return -1;
+      }
+    }
     if(ncplane_putchar(n, '\n') == EOF){
       return -1;
     }
@@ -401,14 +521,22 @@ drawpalette(struct notcurses* nc){
 }
 
 static int
-infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheight){
+newline_past(struct ncplane* std, struct ncplane* p){
+  if(ncplane_putchar_yx(std, ncplane_abs_y(p) + ncplane_dim_y(p) - 1,
+                        ncplane_abs_x(p) + ncplane_dim_x(p) + 1, '\n') != 1){
+    return -1;
+  }
+  return 0;
+}
+
+static int
+infoplane_notcurses(struct notcurses* nc, const fetched_info* fi,
+                    int planeheight, int nextline){
   const int planewidth = 72;
-  int dimy;
-  int y;
+  unsigned dimy;
   struct ncplane* std = notcurses_stddim_yx(nc, &dimy, NULL);
-  ncplane_cursor_yx(std, &y, NULL);
   struct ncplane_options nopts = {
-    .y = y,
+    .y = nextline,
     .x = NCALIGN_CENTER,
     .rows = planeheight,
     .cols = planewidth,
@@ -430,17 +558,17 @@ infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheigh
 #if defined(__linux__)
   struct sysinfo sinfo;
   sysinfo(&sinfo);
-  char totalmet[BPREFIXSTRLEN + 1], usedmet[BPREFIXSTRLEN + 1];
-  bprefix(sinfo.totalram, 1, totalmet, 1);
-  bprefix(sinfo.totalram - sinfo.freeram, 1, usedmet, 1);
+  char totalmet[NCBPREFIXSTRLEN + 1], usedmet[NCBPREFIXSTRLEN + 1];
+  ncbprefix(sinfo.totalram, 1, totalmet, 1);
+  ncbprefix(sinfo.totalram - sinfo.freeram, 1, usedmet, 1);
   ncplane_printf_aligned(infop, 2, NCALIGN_RIGHT, "Processes: %hu ", sinfo.procs);
   ncplane_printf_aligned(infop, 2, NCALIGN_LEFT, " RAM: %sB/%sB", usedmet, totalmet);
 #elif defined(BSD)
   uint64_t ram;
   size_t oldlenp = sizeof(ram);
   if(sysctlbyname("hw.memsize", &ram, &oldlenp, NULL, 0) == 0){
-    char tram[BPREFIXSTRLEN + 1];
-    bprefix(ram, 1, tram, 1);
+    char tram[NCBPREFIXSTRLEN + 1];
+    ncbprefix(ram, 1, tram, 1);
     ncplane_printf_aligned(infop, 2, NCALIGN_LEFT, " RAM: %sB", tram);
   }
 #endif
@@ -448,7 +576,7 @@ infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheigh
   ncplane_printf_aligned(infop, 3, NCALIGN_RIGHT, "Shell: %s ", fi->shell ? fi->shell : "n/a");
   if(notcurses_cantruecolor(nc)){
     ncplane_printf_aligned(infop, 4, NCALIGN_LEFT, " RGB TERM: %s", fi->term);
-    nccell c = CELL_CHAR_INITIALIZER('R');
+    nccell c = NCCELL_CHAR_INITIALIZER('R');
     nccell_set_styles(&c, NCSTYLE_BOLD);
     nccell_set_fg_rgb8(&c, 0xf0, 0xa0, 0xa0);
     ncplane_putc_yx(infop, 4, 1, &c);
@@ -462,18 +590,14 @@ infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheigh
   }else{
     ncplane_printf_aligned(infop, 4, NCALIGN_LEFT, " TERM: %s", fi->term);
   }
-  ncplane_printf_aligned(infop, 4, NCALIGN_RIGHT, "Screen0: %dx%d ", fi->dimx, fi->dimy);
-  ncplane_printf_aligned(infop, 5, NCALIGN_LEFT, " LANG: %s", fi->lang);
-#ifndef __MINGW64__
-  ncplane_printf_aligned(infop, 5, NCALIGN_RIGHT, "UID: %ju ", (uintmax_t)getuid());
-#else
-  ncplane_printf_aligned(infop, 5, NCALIGN_RIGHT, "UID: %s ", "FIXME"); // FIXME
-#endif
+  ncplane_printf_aligned(infop, 4, NCALIGN_RIGHT, " LANG: %s ", fi->lang);
   ncplane_set_styles(infop, NCSTYLE_ITALIC | NCSTYLE_BOLD);
-  ncplane_printf_aligned(infop, 6, NCALIGN_CENTER, "%s (%d cores)", fi->cpu_model, fi->core_count);
-  nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-  nccell ll = CELL_TRIVIAL_INITIALIZER, lr = CELL_TRIVIAL_INITIALIZER;
-  nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+  ncplane_printf_aligned(infop, 5, NCALIGN_CENTER, "%s (%d cores)",
+                         fi->cpu_model ? fi->cpu_model : fallback_cpuinfo(),
+                         fi->core_count);
+  nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+  nccell ll = NCCELL_TRIVIAL_INITIALIZER, lr = NCCELL_TRIVIAL_INITIALIZER;
+  nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
   if(nccells_rounded_box(infop, 0, 0, &ul, &ur, &ll, &lr, &hl, &vl)){
     return -1;
   }
@@ -503,6 +627,12 @@ infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheigh
   ncchannels_set_bg_rgb8(&channels, 0x50, 0x50, 0x50);
   ncplane_set_base(infop, " ", 0, channels);
   ncplane_scrollup_child(std, infop);
+  int parend = ncplane_abs_y(std) + ncplane_dim_y(std) - 1; // where parent ends
+  int chend = ncplane_abs_y(infop) + ncplane_dim_y(infop) - 1; // where child ends
+  if(chend > parend){
+    ncplane_move_rel(infop, -(chend - parend), 0);
+  }
+  newline_past(std, infop);
   if(notcurses_render(nc)){
     return -1;
   }
@@ -510,14 +640,14 @@ infoplane_notcurses(struct notcurses* nc, const fetched_info* fi, int planeheigh
 }
 
 static int
-infoplane(struct notcurses* nc, const fetched_info* fi){
-  const int planeheight = 8;
-  int r = infoplane_notcurses(nc, fi, planeheight);
-  r |= notcurses_stop(nc);
+infoplane(struct notcurses* nc, const fetched_info* fi, int nextline){
+  const int planeheight = 7;
+  int r = infoplane_notcurses(nc, fi, planeheight, nextline);
   return r;
 }
 
 struct marshal {
+  int nextline; // line following display plane, where info plane ought be placed
   struct notcurses* nc;
   const distro_info* dinfo;
   const char* logo; // read from /etc/os-release (or builtin), may be NULL
@@ -533,10 +663,10 @@ neologo_present(struct notcurses* nc, const char* nlogo){
   size_t linelen; // length in bytes, including newline
   char** lines = NULL;
   int linecount = 0;
-  for(const char* cur = nlogo ; *cur ; cur += linelen){
+  for(const char* cur = nlogo ; *cur ; cur += linelen + 1){
     const char* nl = strchr(cur, '\n');
     if(nl){
-      linelen = (nl + 1) - cur;
+      linelen = nl - cur;
     }else{
       linelen = strlen(cur);
     }
@@ -547,28 +677,34 @@ neologo_present(struct notcurses* nc, const char* nlogo){
     }
     lines = tmpl;
     lines[linecount++] = strndup(cur, linelen);
-    size_t collen = ncstrwidth(lines[linecount - 1]);
+    if(nl){ // chomp any newline
+      lines[linecount - 1][linelen] = '\0';
+    }
+    size_t collen = ncstrwidth(lines[linecount - 1], NULL, NULL);
     if(collen > maxlinelen){
       maxlinelen = collen;
     }
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   const int leftpad = (dimx - maxlinelen) / 2;
   for(int i = 0 ; i < linecount ; ++i){
-    ncplane_printf(n, "%*.*s%s", leftpad, leftpad, "", lines[i]);
+    int cols = ncplane_printf(n, "%*.*s%s", leftpad, leftpad, "", lines[i]);
+    if(cols >= 0 && (unsigned)cols < dimx){
+	    ncplane_printf(n, "%*.*s", dimx - cols, dimx - cols, "");
+    }
     free(lines[i]);
   }
   free(lines);
   ncplane_set_fg_default(n);
   ncplane_set_styles(n, NCSTYLE_BOLD | NCSTYLE_ITALIC);
   if(notcurses_canopen_images(nc)){
-    ncplane_putstr_aligned(n, -1, NCALIGN_CENTER, "(no image file is known for your distro)");
+    ncplane_putstr_aligned(n, -1, NCALIGN_CENTER, "(no image file is known for your distro)\n");
   }else{
-    ncplane_putstr_aligned(n, -1, NCALIGN_CENTER, "(notcurses was compiled without image support)");
+    ncplane_putstr_aligned(n, -1, NCALIGN_CENTER, "(notcurses was compiled without image support)\n");
   }
   ncplane_off_styles(n, NCSTYLE_BOLD | NCSTYLE_ITALIC);
-  return notcurses_render(nc);
+  return 0;
 }
 
 static void*
@@ -589,7 +725,7 @@ display_thread(void* vmarshal){
       ncv = ncvisual_from_file(m->dinfo->logofile);
     }
     if(ncv){
-      int y;
+      unsigned y;
       ncplane_cursor_yx(notcurses_stdplane_const(m->nc), &y, NULL);
       bool pixeling = false;
       if(notcurses_check_pixel_support(m->nc) >= 1){
@@ -597,6 +733,7 @@ display_thread(void* vmarshal){
       }
       struct ncvisual_options vopts = {
         .n = notcurses_stdplane(m->nc),
+        .y = y,
         .x = NCALIGN_CENTER,
         .blitter = pixeling ? NCBLIT_PIXEL : NCBLIT_3x2,
         .scaling = pixeling ? NCSCALE_NONE : NCSCALE_SCALE_HIRES,
@@ -605,18 +742,19 @@ display_thread(void* vmarshal){
       struct ncplane* iplane = ncvisual_blit(m->nc, ncv, &vopts);
       ncvisual_destroy(ncv);
       if(iplane){
-        int x = ncplane_x(iplane);
-        ncplane_move_yx(iplane, y, x);
         ncplane_scrollup_child(notcurses_stdplane(m->nc), iplane);
+        newline_past(notcurses_stdplane(m->nc), iplane);
         notcurses_render(m->nc);
-        ncplane_cursor_move_yx(notcurses_stdplane(m->nc),
-                               ncplane_abs_y(iplane) + ncplane_dim_y(iplane), 0);
+        m->nextline = ncplane_y(iplane) + ncplane_dim_y(iplane);
         return NULL;
       }
     }
   }
   if(m->neologo){
     if(neologo_present(m->nc, m->neologo) == 0){
+      unsigned nl;
+      ncplane_cursor_yx(notcurses_stdplane(m->nc), &nl, NULL);
+      m->nextline = nl;
       return NULL;
     }
   }
@@ -653,29 +791,30 @@ ncneofetch(struct notcurses* nc){
     .dinfo = fi.distro,
     .logo = fi.logo,
     .neologo = fi.neologo,
+    .nextline = -1,
   };
   pthread_t tid;
   const bool launched = !pthread_create(&tid, NULL, display_thread, &display_marshal);
   fi.hostname = notcurses_hostname();
   fi.username = notcurses_accountname();
   fetch_env_vars(nc, &fi);
-  if(kern != NCNEO_XNU && kern != NCNEO_WINDOWS){
-    fetch_x_props(&fi);
-  }
   if(kern == NCNEO_LINUX){
     fetch_cpu_info(&fi);
+  }else if(kern == NCNEO_WINDOWS){
+    fetch_windows_cpuinfo(&fi);
   }else{
     fetch_bsd_cpuinfo(&fi);
   }
   if(launched){
     pthread_join(tid, NULL);
   }
-  if(infoplane(nc, &fi)){
+  assert(display_marshal.nextline >= 0);
+  if(infoplane(nc, &fi, display_marshal.nextline)){
     free_fetched_info(&fi);
     return -1;
   }
   free_fetched_info(&fi);
-  return 0;
+  return notcurses_stop(nc);
 }
 
 static void
