@@ -46,8 +46,9 @@ tinfo_debug_style(struct ncplane* n, const char* name, int style, char ch){
 
 static void
 wviz(struct ncplane* n, const wchar_t* wp){
-  for(const wchar_t* w = wp ; *w ; ++w){
-    if(ncplane_putwc(n, *w) <= 0){
+  unsigned wchars;
+  for(const wchar_t* w = wp ; *w ; w += wchars){
+    if(ncplane_putwc_utf32(n, w, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
@@ -92,7 +93,7 @@ braille_viz(struct ncplane* n, wchar_t l, const wchar_t* egcs, wchar_t r,
 
 static void
 finish_line(struct ncplane* n){
-  int x;
+  unsigned x;
   ncplane_cursor_yx(n, NULL, &x);
   while(x++ < 80){
     ncplane_putchar(n, ' ');
@@ -139,7 +140,7 @@ emoji_viz(struct ncplane* n){
                               "\U0001f93d\U0001f3fc\u200d\u2640\ufe0f" // type-3 woman playing water polo
                               ;
   ncplane_set_bg_rgb(n, 0);
-  int bytes;
+  size_t bytes;
   for(const char* e = emoji ; *e ; e += bytes){
     if(ncplane_putegc(n, e, &bytes) < 0){
       if(ncplane_putchar(n, ' ') < 0){
@@ -147,11 +148,14 @@ emoji_viz(struct ncplane* n){
       }
     }
   }
-  int x;
+  unsigned x;
   ncplane_cursor_yx(n, NULL, &x);
   while(x++ < 80){
     ncplane_putchar(n, ' ');
   }
+  if(ncplane_dim_x(n) > 80){
+    ncplane_putchar(n, '\n');
+  }
   return 0;
 }
 
@@ -160,19 +164,24 @@ static int
 legacy_viz(struct ncplane* n, const char* indent, const wchar_t* eighths,
            const wchar_t* anglesr, const wchar_t* anglesl){
   ncplane_printf(n, "%s ", indent);
-  for(const wchar_t* e = eighths ; *e ; ++e){
-    if(ncplane_putwc(n, *e) <= 0){
+  unsigned wchars;
+  for(const wchar_t* e = eighths ; *e ; e += wchars){
+    if(ncplane_putwc_utf32(n, e, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
   ncplane_putchar(n, ' ');
-  for(const wchar_t* r = anglesr ; *r ; ++r){
-    if(ncplane_putwc(n, *r) <= 0){
+  const wchar_t* r = anglesr;
+  const wchar_t* l = anglesl;
+  while(*r){
+    if(ncplane_putwc_utf32(n, r, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
-    if(ncplane_putwc(n, anglesl[r - anglesr]) <= 0){
+    r += wchars;
+    if(ncplane_putwc_utf32(n, l, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
+    l += wchars;
     ncplane_putchar(n, ' ');
   }
   return 0;
@@ -188,8 +197,9 @@ sex_viz(struct ncplane* n, const wchar_t* sex, wchar_t r, const wchar_t* post){
   if(ncplane_putwc(n, r) <= 0){
     ncplane_putchar(n, ' ');
   }
-  for(const wchar_t* p = post ; *p ; ++p){
-    if(ncplane_putwc(n, *p) <= 0){
+  unsigned wchars = 0;
+  for(const wchar_t* p = post ; *p ; p += wchars){
+    if(ncplane_putwc_utf32(n, p, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
@@ -203,7 +213,7 @@ triviz(struct ncplane* n, const wchar_t* w1, const wchar_t* w2, const wchar_t* w
        const wchar_t* wa, const wchar_t* wb, const wchar_t* wc,
        const wchar_t* wd, const wchar_t* we, const wchar_t* wf,
        const wchar_t* w10, const wchar_t* w11, const wchar_t* w12,
-       const wchar_t* w13, const wchar_t* w14){
+       const wchar_t* w13, const wchar_t* w14, const wchar_t* w15){
   wvizn(n, w1, 2);
   ncplane_putchar(n, ' ');
   wvizn(n, w2, 2);
@@ -222,22 +232,20 @@ triviz(struct ncplane* n, const wchar_t* w1, const wchar_t* w2, const wchar_t* w
   wvizn(n, w9, 2);
   wvizn(n, wa, 1);
   ncplane_putchar(n, ' ');
-  ncplane_putchar(n, ' ');
   wvizn(n, wb, 2);
   wvizn(n, wc, 1);
   ncplane_putchar(n, ' ');
   wvizn(n, wd, 2);
   wvizn(n, we, 1);
   ncplane_putchar(n, ' ');
-  ncplane_putchar(n, ' ');
   wvizn(n, wf, 2);
   wvizn(n, w10, 1);
   ncplane_putchar(n, ' ');
   wvizn(n, w11, 2);
   wvizn(n, w12, 1);
-  ncplane_putchar(n, ' ');
-  wviz(n, w13);
+  wvizn(n, w13, 3); // chess
   wviz(n, w14);
+  wviz(n, w15);
 }
 
 static void
@@ -266,7 +274,9 @@ unicodedumper(struct ncplane* n, const char* indent){
   if(notcurses_canutf8(ncplane_notcurses_const(n))){
     // all NCHALFBLOCKS are contained within NCQUADBLOCKS
     ncplane_printf(n, "%s%ls⎧", indent, NCQUADBLOCKS);
-    sex_viz(n, NCSEXBLOCKS, L'⎫', L"♠♥🯰🯱🯲🯳🯴🯵🯶🯷🯸🯹\u2157\u2158\u2159\u215a\u215b");
+    // 🯰🯱🯲🯳🯴🯵🯶🯷🯸🯹 (on Windows, these will be encoded as UTF-16 surrogate
+    // pairs due to a 16-bit wchar_t.
+    sex_viz(n, NCSEXBLOCKS, L'⎫', L"♠♥" NCSEGDIGITS L"\u2157\u2158\u2159\u215a\u215b");
     vertviz(n, L'⎧', NCEIGHTHSR[0], NCEIGHTHSL[0], L'⎫', L"┌╥─╥─╥┐🭩⎛⎞");
     ncplane_printf(n, "%s╲╿╱ ◨◧ ◪◩ ◖◗ ⫷⫸ ⎩", indent);
     sex_viz(n, &NCSEXBLOCKS[32], L'⎭', L"♦♣\u00bc\u00bd\u00be\u2150\u2151\u2152\u2153\u2154\u2155\u2156\u215c\u215d\u215e\u215f\u2189");
@@ -276,14 +286,14 @@ unicodedumper(struct ncplane* n, const char* indent){
            NCCIRCULARARCSW, NCWHITETRIANGLESW, NCSHADETRIANGLESW, NCBLACKTRIANGLESW,
            NCBOXLIGHTW, &NCBOXLIGHTW[4], NCBOXHEAVYW, &NCBOXHEAVYW[4], NCBOXROUNDW,
            &NCBOXROUNDW[4], NCBOXDOUBLEW, &NCBOXDOUBLEW[4], NCBOXOUTERW, &NCBOXOUTERW[4],
-           L"⩘▵△▹▷▿▽◃◁", NCARROWW);
+           NCCHESSBLACK, L"⩘▵△▹▷▿▽◃◁", NCARROWW);
     vertviz(n, L'⎪', NCEIGHTHSR[2], NCEIGHTHSL[2], L'⎪', L"├─╨╫╨─┤┇⎜⎟");
     ncplane_printf(n, "%s╱╽╲ ", indent);
     triviz(n, &NCWHITESQUARESW[2], &NCWHITECIRCLESW[2], &NCDIAGONALSW[2], &NCDIAGONALSW[6],
            &NCCIRCULARARCSW[2], &NCWHITETRIANGLESW[2], &NCSHADETRIANGLESW[2], &NCBLACKTRIANGLESW[2],
            &NCBOXLIGHTW[2], &NCBOXLIGHTW[5], &NCBOXHEAVYW[2], &NCBOXHEAVYW[5], &NCBOXROUNDW[2],
            &NCBOXROUNDW[5], &NCBOXDOUBLEW[2], &NCBOXDOUBLEW[5], &NCBOXOUTERW[2], &NCBOXOUTERW[5],
-           L"⩗▴⏶⯅▲▸⏵⯈▶", L"▾⏷⯆▼◂⏴⯇◀");
+           &NCCHESSBLACK[3], L"⩗▴⏶⯅▲▸⏵⯈▶", L"▾⏷⯆▼◂⏴⯇◀");
     vertviz(n, L'⎪', NCEIGHTHSR[3], NCEIGHTHSL[3], L'⎪', L"╞═╤╬╤═╡┋⎜⎟");
     braille_viz(n, L'⎡', NCBRAILLEEGCS, L'⎤', indent, L"⎨⎬", NCEIGHTHSR[4], NCEIGHTHSL[4],
                 L"╞╕╘╬╛╒╡┊⎜⎟");
@@ -297,7 +307,8 @@ unicodedumper(struct ncplane* n, const char* indent){
     wviz(n, NCDIGITSSUBW);
     wviz(n, L" ⎛");
     wviz(n, NCEIGHTHSB);
-    wviz(n, L" ⎞⎪🭨🭪⎪╰╯││║╔═╝⊴⊵");
+    // 🭫⎞⎪🭨🭪⎪╰╯││║╔═╝⊴⊵
+    wviz(n, L"\U0001FB6B\u239e⎪🭨🭪⎪╰╯││║╔═╝⊴⊵");
     if(ncplane_dim_x(n) > 80){
       ncplane_putchar(n, '\n');
     }
@@ -305,19 +316,19 @@ unicodedumper(struct ncplane* n, const char* indent){
     wviz(n, NCDIGITSSUPERW);
     wviz(n, L" ⎝");
     wviz(n, NCEIGHTHST);
-    wviz(n, L" ⎠⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩⟃⟄");
+    // 🭩⎠⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩⟃⟄
+    wviz(n, L"\U0001FB69\u23a0⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩⟃⟄");
     if(ncplane_dim_x(n) > 80){
       ncplane_putchar(n, '\n');
     }
     emoji_viz(n);
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
     uint64_t ur = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, 0x1B, 0xd8, 0x8E);
     uint64_t lr = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, 0xdB, 0x18, 0x8E);
     uint64_t ul = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, 0x19, 0x19, 0x70);
     uint64_t ll = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, 0x19, 0x19, 0x70);
-    ncplane_cursor_move_yx(n, y - 15, 0);
-    ncplane_stain(n, y - 1, 79, ul, ur, ll, lr);
+    ncplane_stain(n, y - 16, 0, 16, 80, ul, ur, ll, lr);
     ncplane_set_styles(n, NCSTYLE_BOLD | NCSTYLE_ITALIC);
     ncplane_cursor_move_yx(n, y - 12, 54);
     wviz(n, L"🯁🯂🯃https://notcurses.com");
@@ -328,8 +339,8 @@ unicodedumper(struct ncplane* n, const char* indent){
 
 static int
 display_logo(struct ncplane* n, const char* path){
-  int cpixy, cpixx;
-  ncplane_pixelgeom(n, NULL, NULL, &cpixy, &cpixx, NULL, NULL);
+  unsigned cpixy, cpixx;
+  ncplane_pixel_geom(n, NULL, NULL, &cpixy, &cpixx, NULL, NULL);
   struct ncvisual* ncv = ncvisual_from_file(path);
   if(ncv == NULL){
     return -1;
@@ -338,7 +349,7 @@ display_logo(struct ncplane* n, const char* path){
     ncvisual_destroy(ncv);
     return -1;
   }
-  int y;
+  unsigned y;
   ncplane_cursor_yx(n, &y, NULL);
   struct ncvisual_options vopts = {
     .n = n,
@@ -358,16 +369,19 @@ display_logo(struct ncplane* n, const char* path){
 
 static void
 tinfo_debug_bitmaps(struct ncplane* n, const tinfo* ti, const char* indent){
-  if(!(ti->bg_collides_default & 0x80000000)){
-    if(!(ti->bg_collides_default & 0x01000000)){
-      ncplane_printf(n, "%sdefault background 0x%06lx", indent,
-                    ti->bg_collides_default & 0xfffffful);
-    }else{
-      ncplane_printf(n, "%sdefault background 0x%06lx considered transparent", indent,
-                    ti->bg_collides_default & 0xfffffful);
-    }
+  uint32_t fg = 0;
+  int r = notcurses_default_foreground(ncplane_notcurses(n), &fg);
+  if(r){
+    ncplane_printf(n, "%sno known default fg ", indent);
   }else{
-    ncplane_printf(n, "couldn't detect default background");
+    ncplane_printf(n, "%sdefault fg 0x%06x ", indent, fg);
+  }
+  uint32_t bg = 0;
+  r = notcurses_default_background(ncplane_notcurses(n), &bg);
+  if(r){
+    ncplane_printf(n, "no known default bg");
+  }else{
+    ncplane_printf(n, "default bg 0x%06x", bg);
   }
   finish_line(n);
   ncpixelimpl_e blit = notcurses_check_pixel_support(ncplane_notcurses(n));
@@ -434,16 +448,16 @@ tinfo_debug_styles(const notcurses* nc, struct ncplane* n, const char* indent){
   tinfo_debug_cap(n, "el", get_escape(ti, ESCAPE_EL));
   finish_line(n);
   ncplane_putstr(n, indent);
-  tinfo_debug_cap(n, "utf8", ti->caps.utf8);
-  tinfo_debug_cap(n, "2x1", ti->caps.halfblocks);
-  tinfo_debug_cap(n, "2x2", ti->caps.quadrants);
-  tinfo_debug_cap(n, "3x2", ti->caps.sextants);
-  tinfo_debug_cap(n, "4x2", ti->caps.braille);
+  tinfo_debug_cap(n, "utf8", notcurses_canutf8(nc));
+  tinfo_debug_cap(n, "2x1", notcurses_canhalfblock(nc));
+  tinfo_debug_cap(n, "2x2", notcurses_canquadrant(nc));
+  tinfo_debug_cap(n, "3x2", notcurses_cansextant(nc));
+  tinfo_debug_cap(n, "4x2", notcurses_canbraille(nc));
   tinfo_debug_cap(n, "img", notcurses_canopen_images(nc));
   tinfo_debug_cap(n, "vid", notcurses_canopen_videos(nc));
   tinfo_debug_cap(n, "indn", get_escape(ti, ESCAPE_INDN));
   tinfo_debug_cap(n, "gpm", ti->gpmfd >= 0);
-  tinfo_debug_cap(n, "kbd", ti->kbdlevel > 0);
+  tinfo_debug_cap(n, "kbd", ti->kittykbdsupport);
   finish_line(n);
 }
 
@@ -476,9 +490,9 @@ int main(int argc, const char** argv){
     return EXIT_FAILURE;
   }
   // so that we know whether we're talking to gpm
-  notcurses_mouse_enable(nc);
+  notcurses_mice_enable(nc, NCMICE_ALL_EVENTS);
   const char indent[] = "";
-  int dimx;
+  unsigned dimx;
   struct ncplane* stdn = notcurses_stddim_yx(nc, NULL, &dimx);
   if(dimx < 80){
     ncplane_set_fg_rgb(stdn, 0xff5349);
