@@ -125,6 +125,7 @@ const char* nckeystr(char32_t spkey){
     case NCKEY_EXIT:    return "exit";
     case NCKEY_PRINT:   return "print";
     case NCKEY_REFRESH: return "refresh";
+    case NCKEY_SEPARATOR: return "seperator";
     case NCKEY_CAPS_LOCK: return "caps lock";
     case NCKEY_SCROLL_LOCK: return "scroll lock";
     case NCKEY_NUM_LOCK: return "num lock";
@@ -157,6 +158,8 @@ const char* nckeystr(char32_t spkey){
     case NCKEY_RSUPER: return "right super";
     case NCKEY_RHYPER: return "right hyper";
     case NCKEY_RMETA: return "right meta";
+    case NCKEY_L3SHIFT: return "level 3 shift";
+    case NCKEY_L5SHIFT: return "level 5 shift";
     case NCKEY_MOTION: return "mouse (no buttons pressed)";
     case NCKEY_BUTTON1: return "mouse (button 1)";
     case NCKEY_BUTTON2: return "mouse (button 2)";
@@ -330,7 +333,7 @@ int input_demo(ncpp::NotCurses* nc) {
     if(r < 0x80){
       n->set_fg_rgb8(128, 250, 64);
       if(n->printf("ASCII: [0x%02x (%03d)] '%lc'", r, r,
-                   (wchar_t)(iswprint(r) ? r : printutf8(r))) < 0){
+                   (wint_t)(iswprint(r) ? r : printutf8(r))) < 0){
         break;
       }
     }else{
