@@ -9,6 +9,7 @@
 #include "visual-details.h"
 #include "notcurses/direct.h"
 #include "internal.h"
+#include "unixsig.h"
 
 // conform to the foreground and background channels of 'channels'
 static int
@@ -842,6 +843,10 @@ ncdirect_stop_minimal(void* vnc){
         if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
           ret = -1;
         }
+      }else{
+        if(tty_emit(XTMODKEYSUNDO, nc->tcache.ttyfd)){
+          ret = -1;
+        }
       }
     }
     const char* cnorm = get_escape(&nc->tcache, ESCAPE_CNORM);
