@@ -3,6 +3,7 @@
 #include <unistd.h>
 #include "automaton.h"
 #include "internal.h"
+#include "unixsig.h"
 #include "render.h"
 #include "in.h"
 
@@ -100,6 +101,11 @@ typedef struct inputctx {
 
   ipipe ipipes[2];
   ipipe readypipes[2]; // pipes[0]: poll()able fd indicating the presence of user input
+  // initially, initdata is non-NULL and initdata_complete is NULL. once we
+  // get DA1, initdata_complete is non-NULL (it is the same value as
+  // initdata). once we complete reading the input payload that the DA1 arrived
+  // in, initdata is set to NULL, and we broadcast availability. once it has
+  // been taken, both become NULL.
   struct initial_responses* initdata;
   struct initial_responses* initdata_complete;
   bool failed;          // error initializing input automaton, abort
@@ -463,7 +469,7 @@ load_ncinput(inputctx* ictx, const ncinput *tni, int synthsig){
   pthread_mutex_lock(&ictx->ilock);
   if(ictx->ivalid == ictx->isize){
     pthread_mutex_unlock(&ictx->ilock);
-    logerror("dropping input 0x%08x\n", tni->id);
+    logwarn("dropping input 0x%08x\n", tni->id);
     inc_input_errors(ictx);
     send_synth_signal(synthsig);
     return;
@@ -630,7 +636,7 @@ kitty_functional(uint32_t val){
       val = NCKEY_MEDIA_PLAY + val - 57428;
     }else if(val >= 57399 && val <= 57408){
       val = '0' + val - 57399;
-    }else if(val >= 57441 && val <= 57452){
+    }else if(val >= 57441 && val <= 57454){ // up through NCKEY_L5SHIFT
       val = NCKEY_LSHIFT + val - 57441;
     }else switch(val){
       case 57358: val = NCKEY_CAPS_LOCK; break;
@@ -647,6 +653,7 @@ kitty_functional(uint32_t val){
       case 57413: val = '+'; break;
       case 57414: val = NCKEY_ENTER; break;
       case 57415: val = '='; break;
+      case 57416: val = NCKEY_SEPARATOR; break;
       case 57417: val = NCKEY_LEFT; break;
       case 57418: val = NCKEY_RIGHT; break;
       case 57419: val = NCKEY_UP; break;
@@ -994,13 +1001,28 @@ xtsmgraphics_sixel_cb(inputctx* ictx){
 }
 
 static void
-handoff_initial_responses(inputctx* ictx){
+handoff_initial_responses_late(inputctx* ictx){
+  bool sig = false;
   pthread_mutex_lock(&ictx->ilock);
+  if(ictx->initdata_complete && ictx->initdata){
+     ictx->initdata = NULL;
+     sig = true;
+  }
+  pthread_mutex_unlock(&ictx->ilock);
+  if(sig){
+    pthread_cond_broadcast(&ictx->icond);
+    loginfo("handing off initial responses\n");
+  }
+}
+
+
+// mark the initdata as complete, but don't yet broadcast it off.
+static void
+handoff_initial_responses_early(inputctx* ictx){
+  pthread_mutex_lock(&ictx->ilock);
+  // set initdata_complete, but don't clear initdata
   ictx->initdata_complete = ictx->initdata;
-  ictx->initdata = NULL;
   pthread_mutex_unlock(&ictx->ilock);
-  pthread_cond_broadcast(&ictx->icond);
-  loginfo("handing off initial responses\n");
 }
 
 // if XTSMGRAPHICS responses were provided, but DA1 didn't advertise sixel
@@ -1026,7 +1048,7 @@ da1_vt102_cb(inputctx* ictx){
     if(ictx->initdata->qterm != TERMINAL_ALACRITTY){
       scrub_sixel_responses(ictx->initdata);
     }
-    handoff_initial_responses(ictx);
+    handoff_initial_responses_early(ictx);
   }
   return 1;
 }
@@ -1036,7 +1058,7 @@ da1_cb(inputctx* ictx){
   loginfo("read primary device attributes\n");
   if(ictx->initdata){
     scrub_sixel_responses(ictx->initdata);
-    handoff_initial_responses(ictx);
+    handoff_initial_responses_early(ictx);
   }
   return 1;
 }
@@ -1073,7 +1095,7 @@ da1_attrs_cb(inputctx* ictx){
     if(!foundsixel){
       scrub_sixel_responses(ictx->initdata);
     }
-    handoff_initial_responses(ictx);
+    handoff_initial_responses_early(ictx);
   }
   free(attrlist);
   return 1;
@@ -1198,6 +1220,27 @@ decrpm_asu_cb(inputctx* ictx){
   return 2;
 }
 
+static int
+get_default_color(const char* str, uint32_t* color){
+  int r, g, b;
+  if(sscanf(str, "%02x/%02x/%02x", &r, &g, &b) == 3){
+    // great! =]
+  }else if(sscanf(str, "%04x/%04x/%04x", &r, &g, &b) == 3){
+    r /= 256;
+    g /= 256;
+    b /= 256;
+  }else{
+    logerror("couldn't extract rgb from %s\n", str);
+    return -1;
+  }
+  if(r < 0 || g < 0 || b < 0){
+    logerror("invalid colors %d %d %d\n", r, g, b);
+    return -1;
+  }
+  *color = (r << 16u) | (g << 8u) | b;
+  return 0;
+}
+
 static int
 bgdef_cb(inputctx* ictx){
   if(ictx->initdata){
@@ -1205,20 +1248,49 @@ bgdef_cb(inputctx* ictx){
     if(str == NULL){
       logerror("empty bg string\n");
     }else{
-      int r, g, b;
-      if(sscanf(str, "%02x/%02x/%02x", &r, &g, &b) == 3){
-        // great! =]
-      }else if(sscanf(str, "%04x/%04x/%04x", &r, &g, &b) == 3){
-        r /= 256;
-        g /= 256;
-        b /= 256;
-      }else{
-        logerror("couldn't extract rgb from %s\n", str);
-        r = g = b = 0;
+      if(get_default_color(str, &ictx->initdata->bg) == 0){
+        ictx->initdata->got_bg = true;
+        loginfo("default background 0x%06x\n", ictx->initdata->bg);
+      }
+      free(str);
+    }
+  }
+  return 2;
+}
+
+static int
+fgdef_cb(inputctx* ictx){
+  if(ictx->initdata){
+    char* str = amata_next_string(&ictx->amata, "\x1b]10;rgb:");
+    if(str == NULL){
+      logerror("empty fg string\n");
+    }else{
+      if(get_default_color(str, &ictx->initdata->fg) == 0){
+        ictx->initdata->got_fg = true;
+        loginfo("default foreground 0x%06x\n", ictx->initdata->fg);
+      }
+      free(str);
+    }
+  }
+  return 2;
+}
+
+static int
+palette_cb(inputctx* ictx){
+  if(ictx->initdata){
+    unsigned idx = amata_next_numeric(&ictx->amata, "\x1b]4;", ';');
+    char* str = amata_next_string(&ictx->amata, "rgb:");
+    if(idx > sizeof(ictx->initdata->palette.chans) / sizeof(*ictx->initdata->palette.chans)){
+      logerror("invalid index %u\n", idx);
+    }else if(str == NULL){
+      logerror("empty palette string\n");
+    }else{
+      if(get_default_color(str, &ictx->initdata->palette.chans[idx]) == 0){
+        if((int)idx > ictx->initdata->maxpaletteread){
+          ictx->initdata->maxpaletteread = idx;
+        }
+        loginfo("index %u 0x%06x\n", idx, ictx->initdata->palette.chans[idx]);
       }
-      ictx->initdata->bg = (r << 16u) | (g << 8u) | b;
-      ictx->initdata->got_bg = true;
-      loginfo("default background 0x%02x%02x%02x\n", r, g, b);
       free(str);
     }
   }
@@ -1446,6 +1518,7 @@ build_cflow_automaton(inputctx* ictx){
     // OSC (\e_...ST)
     { "_G\\S", kittygraph_cb, },
     // a mystery to everyone!
+    { "]10;rgb:\\S", fgdef_cb, },
     { "]11;rgb:\\S", bgdef_cb, },
     { NULL, NULL, },
   }, *csi;
@@ -1456,6 +1529,17 @@ build_cflow_automaton(inputctx* ictx){
     }
     loginfo("added %p via %s\n", csi->fxn, csi->cflow);
   }
+  if(ictx->ti->qterm == TERMINAL_RXVT){
+    if(inputctx_add_cflow(&ictx->amata, "]4;\\N;rgb:\\R", palette_cb)){
+      logerror("failed adding palette_cb\n");
+      return -1;
+    }
+  }else{
+    if(inputctx_add_cflow(&ictx->amata, "]4;\\N;rgb:\\S", palette_cb)){
+      logerror("failed adding palette_cb\n");
+      return -1;
+    }
+  }
   return 0;
 }
 
@@ -1539,6 +1623,7 @@ create_inputctx(tinfo* ti, FILE* infp, int lmargin, int tmargin, int rmargin,
                             i->initdata->qterm = ti->qterm;
                             i->initdata->cursory = -1;
                             i->initdata->cursorx = -1;
+                            i->initdata->maxpaletteread = -1;
                             i->iread = i->iwrite = i->ivalid = 0;
                             i->cread = i->cwrite = i->cvalid = 0;
                             i->initdata->kbdlevel = UINT_MAX;
@@ -1604,8 +1689,7 @@ free_inputctx(inputctx* i){
     if(i->initdata){
       free(i->initdata->version);
       free(i->initdata);
-    }
-    if(i->initdata_complete){
+    }else if(i->initdata_complete){
       free(i->initdata_complete->version);
       free(i->initdata_complete);
     }
@@ -1741,15 +1825,14 @@ read_input_nblock(int fd, unsigned char* buf, size_t buflen, int *bufused,
 // are terminal and stdin distinct for this inputctx?
 static inline bool
 ictx_independent_p(const inputctx* ictx){
-  return ictx->termfd >= 0; // FIXME does this hold on MSFT Terminal?
+  return ictx->termfd >= 0;
 }
 
 // try to lex a single control sequence off of buf. return the number of bytes
-// consumed if we do so, and -1 otherwise. buf is almost certainly *not*
-// NUL-terminated. if we are definitely *not* an escape, or we're unsure when
-// we run out of input, return the negated relevant number of bytes, setting
-// ictx->midescape if we're uncertain. we preserve a->used, a->state, etc.
-// across runs to avoid reprocessing.
+// consumed if we do so. otherwise, return the negative number of bytes
+// examined. set ictx->midescape if we're uncertain. we preserve a->used,
+// a->state, etc. across runs to avoid reprocessing. buf is almost certainly
+// *not* NUL-terminated.
 //
 // our rule is: an escape must arrive as a single unit to be interpreted as
 // an escape. this is most relevant for Alt+keypress (Esc followed by the
@@ -1815,16 +1898,12 @@ process_escapes(inputctx* ictx, unsigned char* buf, int* bufused){
   int offset = 0;
   while(*bufused){
     int consumed = process_escape(ictx, buf + offset, *bufused);
-    // if we aren't certain, that's not a control sequence unless we're at
-    // the end of the tbuf, in which case we really do try reading more. if
-    // this was not a sequence, we'll catch it on the next read.
+    // negative |consumed| means either that we're not sure whether it's an
+    // escape, or it definitely is not.
     if(consumed < 0){
-      int tavailable = sizeof(ictx->tbuf) - (offset + *bufused - consumed);
-      // if midescape is not set, the negative return menas invalid escape. if
-      // there was space available, we needn't worry about this escape having
-      // been broken across distinct reads. in either case, replay it to the
-      // bulk input buffer; our automaton will have been reset.
-      if(!ictx->midescape || tavailable){
+      // if midescape is not set, the negative return means invalid escape.
+      // replay it to the bulk input buffer; our automaton will have been reset.
+      if(!ictx->midescape){
         consumed = -consumed;
         int available = sizeof(ictx->ibuf) - ictx->ibufvalid;
         if(available){
@@ -1835,15 +1914,17 @@ process_escapes(inputctx* ictx, unsigned char* buf, int* bufused){
           memcpy(ictx->ibuf + ictx->ibufvalid, buf + offset, available);
           ictx->ibufvalid += available;
         }
-        *bufused -= consumed;
         offset += consumed;
         ictx->midescape = 0;
+        *bufused -= consumed;
+        assert(0 <= *bufused);
       }else{
         break;
       }
     }
     *bufused -= consumed;
     offset += consumed;
+    assert(0 <= *bufused);
   }
   // move any leftovers to the front; only happens if we fill output queue,
   // or ran out of input data mid-escape
@@ -1943,6 +2024,7 @@ process_bulk(inputctx* ictx, unsigned char* buf, int* bufused){
     *bufused -= consumed;
     offset += consumed;
   }
+  handoff_initial_responses_late(ictx);
   // move any leftovers to the front
   if(*bufused){
     memmove(buf, buf + offset, *bufused);
@@ -1958,14 +2040,14 @@ static void
 process_melange(inputctx* ictx, const unsigned char* buf, int* bufused){
   int offset = 0;
   while(*bufused){
-    logdebug("input %d/%d [0x%02x] (%c)\n", offset, *bufused, buf[offset],
-             isprint(buf[offset]) ? buf[offset] : ' ');
+    logdebug("input %d (%u)/%d [0x%02x] (%c)\n", offset, ictx->amata.used,
+             *bufused, buf[offset], isprint(buf[offset]) ? buf[offset] : ' ');
     int consumed = 0;
     if(buf[offset] == '\x1b'){
       consumed = process_escape(ictx, buf + offset, *bufused);
       if(consumed < 0){
         if(ictx->midescape){
-          if(offset + *bufused - consumed != sizeof(ictx->ibuf)){
+          if(*bufused != -consumed){
             // not at the end; treat it as input. no need to move between
             // buffers; simply ensure we process it as input, and don't mark
             // anything as consumed.
@@ -1985,6 +2067,7 @@ process_melange(inputctx* ictx, const unsigned char* buf, int* bufused){
     *bufused -= consumed;
     offset += consumed;
   }
+  handoff_initial_responses_late(ictx);
 }
 
 // walk the matching automaton from wherever we were.
@@ -2018,6 +2101,9 @@ process_ibuf(inputctx* ictx){
       // move any leftovers to the front
       if(ictx->ibufvalid){
         memmove(ictx->ibuf, ictx->ibuf + valid - ictx->ibufvalid, ictx->ibufvalid);
+        if(ictx->amata.matchstart){
+          ictx->amata.matchstart = ictx->ibuf;
+        }
       }
     }
   }
@@ -2161,10 +2247,12 @@ read_inputs_nblock(inputctx* ictx){
 
 static void*
 input_thread(void* vmarshall){
+  setup_alt_sig_stack();
   inputctx* ictx = vmarshall;
   if(prep_all_keys(ictx) || build_cflow_automaton(ictx)){
     ictx->failed = true;
-    handoff_initial_responses(ictx);
+    handoff_initial_responses_early(ictx);
+    handoff_initial_responses_late(ictx);
   }
   for(;;){
     read_inputs_nblock(ictx);
@@ -2436,7 +2524,7 @@ int notcurses_linesigs_enable(notcurses* n){
 struct initial_responses* inputlayer_get_responses(inputctx* ictx){
   struct initial_responses* iresp;
   pthread_mutex_lock(&ictx->ilock);
-  while(!ictx->initdata_complete){
+  while(ictx->initdata || !ictx->initdata_complete){
     pthread_cond_wait(&ictx->icond, &ictx->ilock);
   }
   iresp = ictx->initdata_complete;
