@@ -48,6 +48,7 @@ typedef enum {
     TERMINAL_ITERM,         // XTVERSION == 'iTerm2 [ver]'
     TERMINAL_TERMINOLOGY,   // TDA: "~~TY"
     TERMINAL_APPLE,         // Terminal.App, determined by TERM_PROGRAM + macOS
+    TERMINAL_RXVT,          // rxvt/urxvt, determined by TERM + UNIX
     TERMINAL_MSTERMINAL,    // Microsoft Windows Terminal
     TERMINAL_MINTTY,        // XTVERSION == 'mintty ver' MinTTY (Cygwin, MSYS2)
 } queried_terminals_e;
@@ -61,7 +62,9 @@ struct initial_responses {
   queried_terminals_e qterm;   // determined terminal
   unsigned kitty_graphics;     // kitty graphics supported
   uint32_t bg;                 // default background
+  uint32_t fg;                 // default foreground
   bool got_bg;                 // have we read default background?
+  bool got_fg;                 // have we read default foreground?
   bool rgb;                    // was RGB DirectColor advertised?
   int pixx;                    // screen geometry in pixels
   int pixy;                    // screen geometry in pixels
@@ -75,6 +78,8 @@ struct initial_responses {
   int sixelx;                  // maximum sixel width
   char* version;               // version string, heap-allocated
   unsigned kbdlevel;           // enabled kitty keyboard functions
+  ncpalette palette;           // palette entries
+  int maxpaletteread;          // maximum palette index read
 };
 
 // Blocking call. Waits until the input thread has processed all responses to
