@@ -19,10 +19,9 @@ extern "C" {
 #endif
 #define ALLOC __attribute__((malloc)) __attribute__((warn_unused_result))
 
-// KEY_EVENT is defined by both ncurses.h and wincon.h. since we don't use
-// either definition, kill it before inclusion of ncurses.h.
+// KEY_EVENT is defined by both ncurses.h (prior to 6.3) and wincon.h. since we
+// don't use either definition, kill it before inclusion of ncurses.h.
 #undef KEY_EVENT
-#include <ncurses.h> // needed for some definitions, see terminfo(3ncurses)
 #include <term.h>
 #include <time.h>
 #include <term.h>
@@ -74,14 +73,14 @@ struct ncvisual_details;
 typedef struct ncplane {
   nccell* fb;            // "framebuffer" of character cells
   int logrow;            // logical top row, starts at 0, add one for each scroll
-  int x, y;              // current cursor location within this plane
+  unsigned x, y;         // current cursor location within this plane
   // ncplane_yx() etc. use coordinates relative to the plane to which this
   // plane is bound, but absx/absy are always relative to the terminal origin.
   // they must thus be translated by any function which moves a parent plane.
   int absx, absy;        // origin of the plane relative to the pile's origin
                          //  also used as left and top margin on resize by
                          //  ncplane_resize_marginalized()
-  int lenx, leny;        // size of the plane, [0..len{x,y}) is addressable
+  unsigned lenx, leny;   // size of the plane, [0..len{x,y}) is addressable
   egcpool pool;          // attached storage pool for UTF-8 EGCs
   uint64_t channels;     // works the same way as cells
 
@@ -113,6 +112,13 @@ typedef struct ncplane {
   int margin_b, margin_r;// bottom and right margins, stored for resize
   bool scrolling;        // is scrolling enabled? always disabled by default
   bool fixedbound;       // are we fixed relative to the parent's scrolling?
+  bool autogrow;         // do we grow to accommodate output?
+
+  // we need to track any widget to which we are bound, so that (1) we don't
+  // end up bound to two widgets and (2) we can clean them up on shutdown
+  // (assuming they weren't explicitly cleaned up by the client).
+  void* widget;            // widget to which we are bound, can be NULL
+  void(*wdestruct)(void*); // widget destructor, NULL iff widget is NULL
 } ncplane;
 
 // current presentation state of the terminal. it is carried across render
@@ -128,6 +134,8 @@ typedef struct rasterstate {
   // modified by: output, cursor moves, clearing the screen (during refresh).
   int y, x;
 
+  const ncplane* lastsrcp; // last source plane (we emit hpa on plane changes)
+
   unsigned lastr;   // foreground rgb, overloaded for palindexed fg
   unsigned lastg;
   unsigned lastb;
@@ -151,9 +159,6 @@ typedef struct rasterstate {
   bool bgpalelidable;
   bool fgdefelidable;
   bool bgdefelidable;
-
-  // need we do a hard cursor update (i.e. did we just emit a pixel graphic)?
-  bool hardcursorpos;
 } rasterstate;
 
 // Tablets are the toplevel entitites within an ncreel. Each corresponds to
@@ -284,17 +289,37 @@ struct crender {
   } s;
 };
 
+// a pile is a collection of planes which will be rendered together. piles are
+// completely distinct with regards to thread-safety; one can always operate
+// concurrently on distinct piles (save rasterizing, of course). material from
+// other piles will be blown away whenever a pile is rasterized. no ordering
+// is meaningful between planes. when a new one is added, or one is destroyed,
+// the pilelock (in struct notcurses) is taken. a pile is destroyed whenever
+// its last plane is destroyed or reparented away. a pile contains a totally-
+// ordered list of piles (ordered on the z axis) and a directed acyclic forest
+// of bound planes, with each root plane rooting a DAG.
+//
+// the geometries are updated each time the pile is rendered. until a pile is
+// rendered, its geometries might be out-of-date with regards to the terminal
+// description in tcache. when it is rendered again, the geometries will be
+// updated, sprixels will have their TAMs rebuilt (if necessary), and each
+// root plane will have its resize callback invoked (possibly invoking its
+// bound planes' resize callbacks in turn).
+//
+// at context start, there is one pile (the standard pile), containing one
+// plane (the standard plane). each ncplane holds a pointer to its pile.
 typedef struct ncpile {
   ncplane* top;               // topmost plane, never NULL
   ncplane* bottom;            // bottommost plane, never NULL
   ncplane* roots;             // head of root plane list
   struct crender* crender;    // array (rows * cols crender objects)
   struct notcurses* nc;       // notcurses context
-  struct ncpile *prev, *next; // circular list
+  struct ncpile *prev, *next; // circular list pointers
   size_t crenderlen;          // size of crender vector
-  int dimy, dimx;             // rows and cols at time of render
+  unsigned dimy, dimx;        // rows and cols at last render/creation
+  unsigned cellpxx, cellpxy;  // cell-pixel geometry at last render/creation
   int scrolls;                // how many real lines need be scrolled at raster
-  sprixel* sprixelcache;      // list of sprixels
+  sprixel* sprixelcache;      // sorted list of sprixels, assembled during paint
 } ncpile;
 
 // the standard pile can be reached through ->stdplane.
@@ -306,6 +331,7 @@ typedef struct notcurses {
 
   // we keep a copy of the last rendered frame. this facilitates O(1)
   // notcurses_at_yx() and O(1) damage detection (at the cost of some memory).
+  // FIXME why isn't this just an ncplane rather than ~10 different members?
   nccell* lastframe;// last rasterized framebuffer, NULL until first raster
   // the last pile we rasterized. NULL until we've rasterized once. might
   // be invalid due to the pile being destroyed; you are only allowed to
@@ -315,8 +341,8 @@ typedef struct notcurses {
   ncpile* last_pile;
   egcpool pool;   // egcpool for lastframe
 
-  int lfdimx;     // dimensions of lastframe, unchanged by screen resize
-  int lfdimy;     // lfdimx/lfdimy are 0 until first rasterization
+  unsigned lfdimx; // dimensions of lastframe, unchanged by screen resize
+  unsigned lfdimy; // lfdimx/lfdimy are 0 until first rasterization
 
   int cursory;    // desired cursor placement according to user.
   int cursorx;    // -1 is don't-care, otherwise moved here after each render.
@@ -327,13 +353,13 @@ typedef struct notcurses {
   FILE* ttyfp;    // FILE* for writing rasterized data
   tinfo tcache;   // terminfo cache
   pthread_mutex_t pilelock; // guards pile list, locks resize in render
-  bool suppress_banner; // from notcurses_options
 
   // desired margins (best-effort only), copied in from notcurses_options
   int margin_t, margin_b, margin_r, margin_l;
   int loglevel;
   ncpalette palette; // 256-indexed palette can be used instead of/with RGB
   bool palette_damage[NCPALETTESIZE];
+  bool touched_palette; // have we ever changed a palette entry?
   uint64_t flags;  // copied from notcurses_options
 } notcurses;
 
@@ -356,6 +382,8 @@ typedef struct blitterargs {
       sprixel* spx;    // sprixel object
       int pxoffy;      // pixel y-offset within origin cell
       int pxoffx;      // pixel x-offset within origin cell
+      int cellpxy;     // targeted cell-pixel geometry. this ought come from the
+      int cellpxx;     // target ncpile, or tcache in Direct Mode
     } pixel;           // for pixels
   } u;
 } blitterargs;
@@ -386,18 +414,12 @@ struct blitset {
 
 #include "blitset.h"
 
-int ncvisual_blitset_geom(const notcurses* nc, const tinfo* tcache,
-                          const struct ncvisual* n,
-                          const struct ncvisual_options* vopts,
-                          int* y, int* x, int* scaley, int* scalex,
-                          int* leny, int* lenx, const struct blitset** blitter);
-
 void reset_stats(ncstats* stats);
 void summarize_stats(notcurses* nc);
 
 void update_raster_stats(const struct timespec* time1, const struct timespec* time0, ncstats* stats);
 void update_render_stats(const struct timespec* time1, const struct timespec* time0, ncstats* stats);
-void update_render_bytes(ncstats* stats, int bytes);
+void update_raster_bytes(ncstats* stats, int bytes);
 void update_write_stats(const struct timespec* time1, const struct timespec* time0, ncstats* stats, int bytes);
 
 void sigwinch_handler(int signo);
@@ -405,7 +427,7 @@ void sigwinch_handler(int signo);
 void init_lang(void);
 
 int reset_term_attributes(const tinfo* ti, fbuf* f);
-
+int reset_term_palette(const tinfo* ti, fbuf* f, unsigned touchedpalette);
 
 // if there were missing elements we wanted from terminfo, bitch about them here
 void warn_terminfo(const notcurses* nc, const tinfo* ti);
@@ -432,7 +454,25 @@ ncplane_stdplane_const(const ncplane* n){
   return notcurses_stdplane_const(ncplane_notcurses_const(n));
 }
 
-// initialize libav
+// set the plane's widget and wdestruct fields, returning non-zero if they're
+// already non-NULL (i.e. if the plane is already bound), unless we pass NULL
+// (which ought be done from the widget destructor, to avoid corecursion).
+static inline int
+ncplane_set_widget(ncplane* n, void* w, void(*wdestruct)(void*)){
+  if(n->widget){
+    if(w){
+      logerror("plane is already bound to a widget\n");
+      return -1;
+    }
+  }else if(w == NULL){
+    return -1;
+  }
+  n->widget = w;
+  n->wdestruct = wdestruct;
+  return 0;
+}
+
+// initialize visualization backend (ffmpeg/oiio)
 int ncvisual_init(int loglevel);
 
 static inline int
@@ -559,7 +599,7 @@ pool_extended_gcluster(const egcpool* pool, const nccell* c){
 }
 
 static inline nccell*
-ncplane_cell_ref_yx(const ncplane* n, int y, int x){
+ncplane_cell_ref_yx(const ncplane* n, unsigned y, unsigned x){
   return &n->fb[nfbcellidx(n, y, x)];
 }
 
@@ -572,12 +612,12 @@ cell_debug(const egcpool* p, const nccell* c){
 
 static inline void
 plane_debug(const ncplane* n, bool details){
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(n, &dimy, &dimx);
   fprintf(stderr, "p: %p dim: %d/%d poolsize: %d\n", n, dimy, dimx, n->pool.poolsize);
   if(details){
-    for(int y = 0 ; y < 1 ; ++y){
-      for(int x = 0 ; x < 10 ; ++x){
+    for(unsigned y = 0 ; y < 1 ; ++y){
+      for(unsigned x = 0 ; x < 10 ; ++x){
         const nccell* c = &n->fb[fbcellidx(y, dimx, x)];
         fprintf(stderr, "[%03d/%03d] ", y, x);
         cell_debug(&n->pool, c);
@@ -586,6 +626,16 @@ plane_debug(const ncplane* n, bool details){
   }
 }
 
+static inline notcurses*
+ncpile_notcurses(ncpile* p){
+  return p->nc;
+}
+
+static inline const notcurses*
+ncpile_notcurses_const(const ncpile* p){
+  return p->nc;
+}
+
 static inline void
 ncpile_debug(const ncpile* p, fbuf* f){
   fbuf_printf(f, "  ************************* %16p pile ****************************\n", p);
@@ -593,7 +643,7 @@ ncpile_debug(const ncpile* p, fbuf* f){
   const ncplane* prev = NULL;
   int planeidx = 0;
   while(n){
-    fbuf_printf(f, "%04d off y: %3d x: %3d geom y: %3d x: %3d curs y: %3d x: %3d %p %.4s\n",
+    fbuf_printf(f, "%04d off y: %3d x: %3d geom y: %3u x: %3u curs y: %3u x: %3u %p %.4s\n",
                 planeidx, n->absy, n->absx, n->leny, n->lenx, n->y, n->x, n, n->name);
     if(n->boundto || n->bnext || n->bprev || n->blist){
       fbuf_printf(f, " bound %p ← %p → %p binds %p\n",
@@ -639,7 +689,7 @@ void sprixel_free(sprixel* s);
 void sprixel_hide(sprixel* s);
 
 // dimy and dimx are cell geometry, not pixel.
-sprixel* sprixel_alloc(const tinfo* ti, ncplane* n, int dimy, int dimx);
+sprixel* sprixel_alloc(ncplane* n, int dimy, int dimx);
 sprixel* sprixel_recycle(ncplane* n);
 int sprite_init(const tinfo* t, int fd);
 int sprite_clear_all(const tinfo* t, fbuf* f);
@@ -691,7 +741,10 @@ sprite_redraw(notcurses* nc, const ncpile* p, sprixel* s, fbuf* f, int y, int x)
       return 0;
     }
     int r = ti->pixel_draw(ti, p, s, f, y, x);
-    nc->rstate.hardcursorpos = true;
+    // different terminals put the cursor at different places following
+    // emission of a bitmap graphic. just reset y/x.
+    nc->rstate.y = -1;
+    nc->rstate.x = -1;
     return r;
   }
 }
@@ -748,6 +801,8 @@ sprite_rebuild(const notcurses* nc, sprixel* s, int ycell, int xcell){
       free(auxvec);
       s->n->tam[idx].auxvector = NULL;
     }
+  }else{
+    return 0;
   }
   // don't upset SPRIXEL_MOVED
   if(s->invalidated == SPRIXEL_QUIESCENT){
@@ -768,7 +823,7 @@ sprite_rebuild(const notcurses* nc, sprixel* s, int ycell, int xcell){
 // happy fact: common reported values for maximum sixel height are 256, 1024,
 // and 4096...not a single goddamn one of which is divisible by six. augh.
 static inline void
-clamp_to_sixelmax(const tinfo* t, int* y, int* x, int* outy, ncscale_e scaling){
+clamp_to_sixelmax(const tinfo* t, unsigned* y, unsigned* x, unsigned* outy, ncscale_e scaling){
   if(t->sixel_maxy && *y > t->sixel_maxy){
     *y = t->sixel_maxy;
   }
@@ -826,9 +881,9 @@ sprixel_state(const sprixel* s, int y, int x){
   int localx = x - (s->n->absx - stdn->absx);
 //fprintf(stderr, "TAM %d at %d/%d (%d/%d, %d/%d)\n", s->n->tam[localy * s->dimx + localx].state, localy, localx, y, x, s->dimy, s->dimx);
   assert(localy >= 0);
-  assert(localy < s->dimy);
+  assert(localy < (int)s->dimy);
   assert(localx >= 0);
-  assert(localx < s->dimx);
+  assert(localx < (int)s->dimx);
   return s->n->tam[localy * s->dimx + localx].state;
 }
 
@@ -869,10 +924,13 @@ cell_duplicate_far(egcpool* tpool, nccell* targ, const ncplane* splane, const nc
 }
 
 int ncplane_resize_internal(ncplane* n, int keepy, int keepx,
-                            int keepleny, int keeplenx, int yoff, int xoff,
-                            int ylen, int xlen);
+                            unsigned keepleny, unsigned keeplenx,
+                            int yoff, int xoff,
+                            unsigned ylen, unsigned xlen);
 
-int update_term_dimensions(int* rows, int* cols, tinfo* tcache, int margin_b);
+int update_term_dimensions(unsigned* rows, unsigned* cols, tinfo* tcache, int margin_b,
+                           unsigned* cgeo_changed, unsigned* pgeo_changed)
+  __attribute__ ((nonnull (3, 5, 6)));
 
 ALLOC static inline void*
 memdup(const void* src, size_t len){
@@ -938,7 +996,7 @@ int ncvisual_bounding_box(const struct ncvisual* ncv, int* leny, int* lenx,
 //  steps / 2 is to work around truncate-towards-zero).
 static int
 calc_gradient_component(unsigned tl, unsigned tr, unsigned bl, unsigned br,
-                        int y, int x, int ylen, int xlen){
+                        unsigned y, unsigned x, unsigned ylen, unsigned xlen){
   const int avm = (ylen - 1) - y;
   const int ahm = (xlen - 1) - x;
   if(xlen < 2){
@@ -961,7 +1019,7 @@ calc_gradient_component(unsigned tl, unsigned tr, unsigned bl, unsigned br,
 // calculate one of the channels of a gradient at a particular point.
 static inline uint32_t
 calc_gradient_channel(uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr,
-                      int y, int x, int ylen, int xlen){
+                      unsigned y, unsigned x, unsigned ylen, unsigned xlen){
   uint32_t chan = 0;
   ncchannel_set_rgb8_clipped(&chan,
                            calc_gradient_component(ncchannel_r(ul), ncchannel_r(ur),
@@ -981,8 +1039,8 @@ calc_gradient_channel(uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr,
 // into `channels'. x and y ought be the location within the gradient.
 static inline void
 calc_gradient_channels(uint64_t* channels, uint64_t ul, uint64_t ur,
-                       uint64_t ll, uint64_t lr, int y, int x,
-                       int ylen, int xlen){
+                       uint64_t ll, uint64_t lr, unsigned y, unsigned x,
+                       unsigned ylen, unsigned xlen){
   if(!ncchannels_fg_default_p(ul)){
     ncchannels_set_fchannel(channels,
                             calc_gradient_channel(ncchannels_fchannel(ul),
@@ -1109,61 +1167,54 @@ coerce_styles(fbuf* f, const tinfo* ti, uint16_t* curstyle,
   return ret;
 }
 
-#define SET_BTN_EVENT_MOUSE   "1002"
-#define SET_FOCUS_EVENT_MOUSE "1004"
-#define SET_SGR_MODE_MOUSE    "1006"
+// DEC private mode set (DECSET) parameters (and corresponding XTerm resources)
+#define SET_X10_MOUSE_PROT       "9" // outdated, do not use, use x11
+// we can combine 1000--1004, and then use 1005/1006/1015 for extended coordinates
+#define SET_X11_MOUSE_PROT    "1000" // for button events
+#define SET_HILITE_MOUSE_PROT "1001" // for highlight tracking
+#define SET_BTN_EVENT_MOUSE   "1002" // for motion events with buttons
+#define SET_ALL_EVENT_MOUSE   "1003" // for motion events without buttons
+#define SET_FOCUS_EVENT_MOUSE "1004" // for focus events
+#define SET_UTF8_MOUSE_PROT   "1005" // utf8-style extended coordinates
+#define SET_SGR_MOUSE_PROT    "1006" // sgr-style extended coordinates
+#define SET_ALTERNATE_SCROLL  "1007" // scroll in alternate screen (alternateScroll)
+#define SET_TTYOUTPUT_SCROLL  "1010" // scroll on tty output (scrollTtyOutput)
+#define SET_KEYPRESS_SCROLL   "1011" // scroll on keypress (scrollKey)
+#define SET_URXVT_MOUSE_PROT  "1015" // urxvt-style extended coordinates
+#define SET_PIXEL_MOUSE_PROT  "1016" // sgr-style, using pixles rather than cells
+#define SET_ENABLE_ALTSCREEN  "1046" // enable alternate screen (*sets* titeInhibit)
+#define SET_ALTERNATE_SCREEN  "1047" // replaces 47 (conflict w/DECGRPM) (titeInhibit)
+#define SET_SAVE_CURSOR       "1048" // save cursor ala DECSC (titeInhibit)
+#define SET_SMCUP             "1049" // 1047+1048 (titeInhibit)
+// DECSET/DECRSTs can be chained with semicolons; can we generalize this? FIXME
+#define DECSET(p) "\x1b[?" p "h"
+#define DECRST(p) "\x1b[?" p "l"
 
-static inline int
-mouse_enable(tinfo* ti, FILE* out){
-  if(ti->qterm == TERMINAL_LINUX){
-    if(ti->gpmfd < 0){
-      if((ti->gpmfd = gpm_connect(ti)) < 0){
-        return -1;
-      }
-    }
-    return 0;
-  }
-// Sets the shift-escape option, allowing shift+mouse to override the standard
-// mouse protocol (mainly so copy-and-paste can still be performed).
-#define XTSHIFTESCAPE "\x1b[>1s"
-  return term_emit(XTSHIFTESCAPE "\x1b[?" SET_BTN_EVENT_MOUSE ";"
-                   /*SET_FOCUS_EVENT_MOUSE ";" */SET_SGR_MODE_MOUSE "h",
-                   out, true);
-#undef XTSHIFTESCAPE
-}
-
-static inline int
-mouse_disable(tinfo* ti, fbuf* f){
-  if(ti->qterm == TERMINAL_LINUX){
-    if(ti->gpmfd < 0){
-      return 0;
-    }
-    ti->gpmfd = -1;
-    return gpm_close(ti);
-  }
-  return fbuf_emit(f, "\x1b[?" SET_BTN_EVENT_MOUSE ";"
-                   /*SET_FOCUS_EVENT_MOUSE ";" */SET_SGR_MODE_MOUSE "l");
-}
+int mouse_setup(tinfo* ti, unsigned eventmask);
 
 // sync the drawing position to the specified location with as little overhead
 // as possible (with nothing, if already at the right location). we prefer
 // absolute horizontal moves (hpa) to relative ones, in the rare event that
-// our understanding of our horizontal location is faulty.
+// our understanding of our horizontal location is faulty. if we're moving from
+// one plane to another, we emit an hpa no matter what.
 // FIXME fall back to synthesized moves in the absence of capabilities (i.e.
 // textronix lacks cup; fake it with horiz+vert moves)
 // if hardcursorpos is non-zero, we always perform a cup. this is done when we
 // don't know where the cursor currently is =].
 static inline int
-goto_location(notcurses* nc, fbuf* f, int y, int x){
+goto_location(notcurses* nc, fbuf* f, int y, int x, const ncplane* srcp){
 //fprintf(stderr, "going to %d/%d from %d/%d hard: %u\n", y, x, nc->rstate.y, nc->rstate.x, nc->rstate.hardcursorpos);
   int ret = 0;
   // if we don't have hpa, force a cup even if we're only 1 char away. the only
   // TERM i know supporting cup sans hpa is vt100, and vt100 can suck it.
   // you can't use cuf for backwards moves anyway; again, vt100 can suck it.
   const char* hpa = get_escape(&nc->tcache, ESCAPE_HPA);
-  if(nc->rstate.y == y && hpa && !nc->rstate.hardcursorpos){ // only need move x
-    if(nc->rstate.x == x){ // needn't move shit
-      return 0;
+  if(nc->rstate.y == y && hpa){ // only need move x
+    if(nc->rstate.x == x){
+      if(nc->rstate.lastsrcp == srcp || !nc->tcache.gratuitous_hpa){
+        return 0; // needn't move shit
+      }
+      ++nc->stats.s.hpa_gratuitous;
     }
     if(fbuf_emit(f, tiparm(hpa, x))){
       return -1;
@@ -1175,13 +1226,9 @@ goto_location(notcurses* nc, fbuf* f, int y, int x){
       return -1;
     }
   }
-  if(y > nc->rstate.logendy || (y == nc->rstate.logendy && x > nc->rstate.logendx)){
-    nc->rstate.logendy = y;
-    nc->rstate.logendx = x;
-  }
   nc->rstate.x = x;
   nc->rstate.y = y;
-  nc->rstate.hardcursorpos = 0;
+  nc->rstate.lastsrcp = srcp;
   return ret;
 }
 
@@ -1194,10 +1241,23 @@ box_corner_needs(unsigned ctlword){
 // True if the cell does not generate background pixels (i.e., the cell is a
 // solid or shaded block, or certain emoji).
 static inline bool
-cell_nobackground_p(const nccell* c){
+nccell_nobackground_p(const nccell* c){
+  // needs to match all four bits of NC_NOBACKGROUND_MASK, not just one
   return (c->channels & NC_NOBACKGROUND_MASK) == NC_NOBACKGROUND_MASK;
 }
 
+// True iff the foreground and background color are both RGB, and equal.
+static inline bool
+nccell_rgbequal_p(const nccell* c){
+  if(nccell_fg_default_p(c) || nccell_fg_palindex_p(c)){
+    return false;
+  }
+  if(nccell_bg_default_p(c) || nccell_bg_palindex_p(c)){
+    return false;
+  }
+  return nccell_fg_rgb(c) == nccell_bg_rgb(c);
+}
+
 // Returns a number 0 <= n <= 15 representing the four quadrants, and which (if
 // any) are occupied due to blitting with a transparent background. The mapping
 // is {tl, tr, bl, br}.
@@ -1277,25 +1337,33 @@ cell_set_fchannel(nccell* cl, uint32_t channel){
 // 'c2'. If 'c1' is otherwise the default color, 'c1' will not be touched,
 // since we can't blend default colors. Likewise, if 'c2' is a default color,
 // it will not be used (unless 'blends' is 0).
-//
-// Palette-indexed colors do not blend. Do not pass me palette-indexed channels!
 static inline unsigned
-channels_blend(unsigned c1, unsigned c2, unsigned* blends){
+channels_blend(notcurses* nc, unsigned c1, unsigned c2, unsigned* blends){
   if(ncchannel_alpha(c2) == NCALPHA_TRANSPARENT){
     return c1; // do *not* increment *blends
   }
   bool c2default = ncchannel_default_p(c2);
+  bool c2palette = ncchannel_palindex_p(c2);
   if(*blends == 0){
     // don't just return c2, or you set wide status and all kinds of crap
-    if(ncchannel_default_p(c2)){
+    if(c2default){
       ncchannel_set_default(&c1);
+    }else if(c2palette){
+      ncchannel_set_palindex(&c1, ncchannel_palindex(c2));
     }else{
       ncchannel_set(&c1, c2 & NC_BG_RGB_MASK);
     }
     ncchannel_set_alpha(&c1, ncchannel_alpha(c2));
   }else if(!c2default && !ncchannel_default_p(c1)){
     unsigned rsum, gsum, bsum;
-    ncchannel_rgb8(c2, &rsum, &gsum, &bsum);
+    if(c2palette){
+      uint32_t rgb = nc->palette.chans[ncchannel_palindex(c2)];
+      bsum = rgb & 0xff;
+      gsum = (rgb >> 8u) & 0xff;
+      rsum = (rgb >> 16u) & 0xff;
+    }else{
+      ncchannel_rgb8(c2, &rsum, &gsum, &bsum);
+    }
     rsum = (ncchannel_r(c1) * *blends + rsum) / (*blends + 1);
     gsum = (ncchannel_g(c1) * *blends + gsum) / (*blends + 1);
     bsum = (ncchannel_b(c1) * *blends + bsum) / (*blends + 1);
@@ -1308,40 +1376,13 @@ channels_blend(unsigned c1, unsigned c2, unsigned* blends){
 
 // do not pass palette-indexed channels!
 static inline uint64_t
-cell_blend_fchannel(nccell* cl, unsigned channel, unsigned* blends){
-  return cell_set_fchannel(cl, channels_blend(cell_fchannel(cl), channel, blends));
+cell_blend_fchannel(notcurses* nc, nccell* cl, unsigned channel, unsigned* blends){
+  return cell_set_fchannel(cl, channels_blend(nc, cell_fchannel(cl), channel, blends));
 }
 
 static inline uint64_t
-cell_blend_bchannel(nccell* cl, unsigned channel, unsigned* blends){
-  return cell_set_bchannel(cl, channels_blend(cell_bchannel(cl), channel, blends));
-}
-
-// examine the UTF-8 EGC in the first |*bytes| bytes of |egc|. if the EGC is
-// right-to-left, we make a copy, appending an U+200E to force left-to-right.
-// only the first unicode char of the EGC is currently checked FIXME. if the
-// EGC is not RTL, we return NULL.
-ALLOC static inline char*
-egc_rtl(const char* egc, int* bytes){
-  wchar_t w;
-  mbstate_t mbstate = { };
-  size_t r = mbrtowc(&w, egc, *bytes, &mbstate);
-  if(r == (size_t)-1 || r == (size_t)-2){
-    return NULL;
-  }
-  const int bidic = uc_bidi_category(w);
-//fprintf(stderr, "BIDI CAT (%lc): %d\n", w, bidic);
-  if(bidic != UC_BIDI_R && bidic != UC_BIDI_AL && bidic != UC_BIDI_RLE && bidic != UC_BIDI_RLO){
-    return NULL;
-  }
-  // insert U+200E, "LEFT-TO-RIGHT MARK". This ought reset the text direction
-  // after emitting a potentially RTL EGC.
-  const char LTRMARK[] = "\xe2\x80\x8e";
-  char* s = (char*)malloc(*bytes + sizeof(LTRMARK)); // cast for C++ callers
-  memcpy(s, egc, *bytes);
-  memcpy(s + *bytes, LTRMARK, sizeof(LTRMARK));
-  *bytes += strlen(LTRMARK);
-  return s;
+cell_blend_bchannel(notcurses* nc, nccell* cl, unsigned channel, unsigned* blends){
+  return cell_set_bchannel(cl, channels_blend(nc, cell_bchannel(cl), channel, blends));
 }
 
 // a sprixel occupies the entirety of its associated plane, usually an entirely
@@ -1394,7 +1435,10 @@ pool_blit_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int
   if(bytes < 0 || cols < 0){
     return -1;
   }
-  if(is_control_egc((const unsigned char*)gcluster, bytes)){
+  // we allow newlines to be blitted into the pool, as they're picked up and
+  // given special semantics by paint() and rasterization.
+  if(*gcluster != '\n' && is_control_egc((const unsigned char*)gcluster, bytes)){
+    logerror("not loading control character %u\n", *(const unsigned char*)gcluster);
     return -1;
   }
   c->width = cols;
@@ -1411,21 +1455,12 @@ pool_blit_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int
   return bytes;
 }
 
-// Do an RTL-check, reset the quadrant occupancy bits, and pass the cell down to
+// Reset the quadrant occupancy bits, and pass the cell down to
 // pool_blit_direct(). Returns the number of bytes loaded.
 static inline int
 pool_load_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int cols){
-  char* rtl = NULL;
   c->channels &= ~NC_NOBACKGROUND_MASK;
-  if(bytes >= 0){
-    rtl = egc_rtl(gcluster, &bytes); // checks for RTL and adds U+200E if so
-  }
-  if(rtl){
-    gcluster = rtl;
-  }
-  int r = pool_blit_direct(pool, c, gcluster, bytes, cols);
-  free(rtl);
-  return r;
+  return pool_blit_direct(pool, c, gcluster, bytes, cols);
 }
 
 static inline int
@@ -1485,13 +1520,68 @@ int get_tty_fd(FILE* ttyfp);
 // If you only want to check n < 4 channels, just duplicate one.
 bool check_gradient_args(uint64_t ul, uint64_t ur, uint64_t bl, uint64_t br);
 
-int setup_signals(void* nc, bool no_quit_sigs, bool no_winch_sig,
-                  int(*handler)(void*));
-int drop_signals(void* nc);
-
-// block a few signals for the duration of a write to the terminal.
-int block_signals(sigset_t* old_blocked_signals);
-int unblock_signals(const sigset_t* old_blocked_signals);
+// takes a signed starting coordinate (where -1 indicates the cursor's
+// position), and an unsigned vector (where 0 indicates "everything
+// remaining", i.e. to the right and below). returns 0 iff everything
+// is valid and on the plane, filling in 'ystart'/'xstart' with the
+// (non-negative) starting coordinates and 'ylen'/'xlen with the
+// (positive) dimensions of the affected area.
+static inline int
+check_geometry_args(const ncplane* n, int y, int x,
+                    unsigned* ylen, unsigned* xlen,
+                    unsigned* ystart, unsigned* xstart){
+  // handle the special -1 case for y/x, and reject other negatives
+  if(y < 0){
+    if(y != -1){
+      logerror("invalid y: %d\n", y);
+      return -1;
+    }
+    y = n->y;
+  }
+  if(x < 0){
+    if(x != -1){
+      logerror("invalid x: %d\n", x);
+      return -1;
+    }
+    x = n->x;
+  }
+  // y and x are both now definitely positive, but might be off-plane.
+  // lock in y and x as ystart and xstart for unsigned comparisons.
+  *ystart = y;
+  *xstart = x;
+  unsigned ymax, xmax;
+  ncplane_dim_yx(n, &ymax, &xmax);
+  if(*ystart >= ymax || *xstart >= xmax){
+    logerror("invalid starting coordinates: %u/%u\n", *ystart, *xstart);
+    return -1;
+  }
+  // handle the special 0 case for ylen/xlen
+  if(*ylen == 0){
+    *ylen = ymax - *ystart;
+  }
+  if(*xlen == 0){
+    *xlen = xmax - *xstart;
+  }
+  // ensure ylen/xlen are on-plane
+  if(*ylen > ymax){
+    logerror("ylen > dimy %u > %u\n", *ylen, ymax);
+    return -1;
+  }
+  if(*xlen > xmax){
+    logerror("xlen > dimx %u > %u\n", *xlen, xmax);
+    return -1;
+  }
+  // ensure x + xlen and y + ylen are on-plane, without overflow
+  if(ymax - *ylen < *ystart){
+    logerror("y + ylen > ymax %u + %u > %u\n", *ystart, *ylen, ymax);
+    return -1;
+  }
+  if(xmax - *xlen < *xstart){
+    logerror("x + xlen > xmax %u + %u > %u\n", *xstart, *xlen, xmax);
+    return -1;
+  }
+  return 0;
+}
 
 void ncvisual_printbanner(fbuf* f);
 
@@ -1506,7 +1596,7 @@ rgba_trans_p(uint32_t p, uint32_t transcolor){
   if(ncpixel_a(p) < 192){
     return true;
   }
-  if(transcolor && 
+  if(transcolor &&
       (ncpixel_r(p) == (transcolor & 0xff0000ull) >> 16) &&
       (ncpixel_g(p) == (transcolor & 0xff00ull) >> 8) &&
       (ncpixel_b(p) == (transcolor & 0xffull))){
@@ -1590,6 +1680,13 @@ rgba_blit_dispatch(ncplane* nc, const struct blitset* bset,
   return bset->blit(nc, linesize, data, leny, lenx, bargs);
 }
 
+int ncvisual_geom_inner(const tinfo* ti, const struct ncvisual* n,
+                        const struct ncvisual_options* vopts, ncvgeom* geom,
+                        const struct blitset** bset,
+                        unsigned* disppxy, unsigned* disppxx,
+                        unsigned* outy, unsigned* outx,
+                        int* placey, int* placex);
+
 static inline const struct blitset*
 rgba_blitter_low(const tinfo* tcache, ncscale_e scale, bool maydegrade,
                  ncblitter_e blitrec) {
@@ -1653,12 +1750,33 @@ resize_bitmap(const uint32_t* bmap, int srows, int scols, size_t sstride,
   return ret;
 }
 
+// a neighbor on which to polyfill. by the time we get to it, it might or
+// might not have been filled in. if so, discard immediately. otherwise,
+// check self, and if valid, push all neighbors.
+struct topolyfill {
+  int y, x;
+  struct topolyfill* next;
+};
+
+static inline struct topolyfill*
+create_polyfill_op(int y, int x, struct topolyfill** stck){
+  // cast for the benefit of c++ callers
+  struct topolyfill* n = (struct topolyfill*)malloc(sizeof(*n));
+  if(n){
+    n->y = y;
+    n->x = x;
+    n->next = *stck;
+    *stck = n;
+  }
+  return n;
+}
+
 // implemented by a multimedia backend (ffmpeg or oiio), and installed
 // prior to calling notcurses_core_init() (by notcurses_init()).
 typedef struct ncvisual_implementation {
   int (*visual_init)(int loglevel);
   void (*visual_printbanner)(fbuf* f);
-  int (*visual_blit)(struct ncvisual* ncv, int rows, int cols, ncplane* n,
+  int (*visual_blit)(struct ncvisual* ncv, unsigned rows, unsigned cols, ncplane* n,
                      const struct blitset* bset, const blitterargs* barg);
   struct ncvisual* (*visual_create)(void);
   struct ncvisual* (*visual_from_file)(const char* fname);
@@ -1670,26 +1788,17 @@ typedef struct ncvisual_implementation {
   int (*visual_decode_loop)(struct ncvisual* nc);
   int (*visual_stream)(notcurses* nc, struct ncvisual* ncv, float timescale,
                        ncstreamcb streamer, const struct ncvisual_options* vopts, void* curry);
-  struct ncplane* (*visual_subtitle)(struct ncplane* parent, const struct ncvisual* ncv);
+  ncplane* (*visual_subtitle)(ncplane* parent, const struct ncvisual* ncv);
   int rowalign; // rowstride base, can be 0 for no padding
   // do a persistent resize, changing the ncv itself
-  int (*visual_resize)(struct ncvisual* ncv, int rows, int cols);
+  int (*visual_resize)(struct ncvisual* ncv, unsigned rows, unsigned cols);
   void (*visual_destroy)(struct ncvisual* ncv);
   bool canopen_images;
   bool canopen_videos;
 } ncvisual_implementation;
 
 // populated by libnotcurses.so if linked with multimedia
-API extern ncvisual_implementation visual_implementation;
-
-static inline char
-path_separator(void){
-#ifdef __MINGW64__
-  return '\\';
-#else
-  return '/';
-#endif
-}
+API extern ncvisual_implementation* visual_implementation;
 
 // prepend base with the Notcurses data directory as configured.
 static inline char*
@@ -1713,7 +1822,7 @@ prefix_data(const char* base){
 static inline int
 tty_check(int fd){
 #ifdef __MINGW64__
-  return fd >= 0 ? 1 : 0;
+  return _isatty(fd);
 #endif
   return isatty(fd);
 }
@@ -1734,6 +1843,7 @@ cancel_and_join(const char* name, pthread_t tid, void** res){
 
 static inline int
 emit_scrolls(const tinfo* ti, int count, fbuf* f){
+  logdebug("emitting %d scrolls\n", count);
   if(count > 1){
     const char* indn = get_escape(ti, ESCAPE_INDN);
     if(indn){
@@ -1747,6 +1857,7 @@ emit_scrolls(const tinfo* ti, int count, fbuf* f){
   if(ind == NULL){
     ind = "\v";
   }
+  // fell through if we had no indn
   while(count > 0){
     if(fbuf_emit(f, ind) < 0){
       return -1;
@@ -1756,6 +1867,18 @@ emit_scrolls(const tinfo* ti, int count, fbuf* f){
   return 0;
 }
 
+// both emit the |count > 0| scroll ops to |f|, and update the cursor
+// tracking in |nc|
+static inline int
+emit_scrolls_track(notcurses* nc, int count, fbuf* f){
+  if(emit_scrolls(&nc->tcache, count, f)){
+    return -1;
+  }
+  nc->rstate.y -= count;
+  nc->rstate.x = 0;
+  return 0;
+}
+
 // replace or populate the TERM environment variable with 'termname'
 int putenv_term(const char* termname) __attribute__ ((nonnull (1)));
 
