@@ -16,6 +16,7 @@
 #include <inttypes.h>
 #include <notcurses/direct.h>
 #include "compat/compat.h"
+#include "unixsig.h"
 #include "banner.h"
 
 #define ESC "\x1b"
@@ -28,7 +29,10 @@ void notcurses_version_components(int* major, int* minor, int* patch, int* tweak
 }
 
 int notcurses_enter_alternate_screen(notcurses* nc){
-  if(enter_alternate_screen(nc->ttyfp, &nc->tcache, true, nc->flags & NCOPTION_DRAIN_INPUT)){
+  if(nc->tcache.ttyfd < 0){
+    return -1;
+  }
+  if(enter_alternate_screen(nc->tcache.ttyfd, nc->ttyfp, &nc->tcache, nc->flags & NCOPTION_DRAIN_INPUT)){
     return -1;
   }
   ncplane_set_scrolling(notcurses_stdplane(nc), false);
@@ -36,7 +40,11 @@ int notcurses_enter_alternate_screen(notcurses* nc){
 }
 
 int notcurses_leave_alternate_screen(notcurses* nc){
-  if(leave_alternate_screen(nc->ttyfp, &nc->tcache, nc->flags & NCOPTION_DRAIN_INPUT)){
+  if(nc->tcache.ttyfd < 0){
+    return -1;
+  }
+  if(leave_alternate_screen(nc->tcache.ttyfd, nc->ttyfp,
+                            &nc->tcache, nc->flags & NCOPTION_DRAIN_INPUT)){
     return -1;
   }
   // move to the end of our output
@@ -58,8 +66,49 @@ int reset_term_attributes(const tinfo* ti, fbuf* f){
   if((esc = get_escape(ti, ESCAPE_SGR0)) && fbuf_emit(f, esc)){
     ret = -1;
   }
-  if((esc = get_escape(ti, ESCAPE_OC)) && fbuf_emit(f, esc)){
-    ret = -1;
+  return ret;
+}
+
+// attempt to restore the palette. if XT{PUSH,POP}COLORS is supported, use
+// XTPOPCOLORS. if we can program individual colors, and we read the palette,
+// reload it from our initial capture. otherwise, use "oc" if available; this
+// will blow away any preexisting palette in favor of the default. if we've
+// never touched the palette, don't bother trying to restore it (unless we're
+// using XTPOPCOLORS, since in that case we always used XTPUSHCOLORS).
+int reset_term_palette(const tinfo* ti, fbuf* f, unsigned touchedpalette){
+  int ret = 0;
+  const char* esc;
+  if((esc = get_escape(ti, ESCAPE_RESTORECOLORS))){
+    loginfo("restoring palette via xtpopcolors\n");
+    if(fbuf_emit(f, esc)){
+      ret = -1;
+    }
+    return ret;
+  }
+  if(!touchedpalette){
+    return 0;
+  }
+  if(ti->caps.can_change_colors && ti->maxpaletteread > -1){
+    loginfo("restoring saved palette (%d)\n", ti->maxpaletteread + 1);
+    esc = get_escape(ti, ESCAPE_INITC);
+    for(int z = 0 ; z < ti->maxpaletteread ; ++z){
+      unsigned r, g, b;
+      ncchannel_rgb8(ti->originalpalette.chans[z], &r, &g, &b);
+      // Need convert RGB values [0..256) to [0..1000], ugh
+      r = r * 1000 / 255;
+      g = g * 1000 / 255;
+      b = b * 1000 / 255;
+      if(fbuf_emit(f, tiparm(esc, z, r, g, b)) < 0){
+        return -1;
+      }
+    }
+  }else if((esc = get_escape(ti, ESCAPE_OC))){
+    loginfo("resetting palette\n");
+    if(fbuf_emit(f, esc)){
+      ret = -1;
+    }
+  }else{
+    logwarn("no method known to restore palette\n");
   }
   return ret;
 }
@@ -80,9 +129,7 @@ notcurses_stop_minimal(void* vnc){
   // be sure to write the restoration sequences *prior* to running rmcup, as
   // they apply to the screen (alternate or otherwise) we're actually using.
   const char* esc;
-  if((esc = get_escape(&nc->tcache, ESCAPE_RESTORECOLORS)) && fbuf_emit(f, esc)){
-    ret = -1;
-  }
+  ret |= reset_term_palette(&nc->tcache, f, nc->touched_palette);
   ret |= reset_term_attributes(&nc->tcache, f);
   if((esc = get_escape(&nc->tcache, ESCAPE_RMKX)) && fbuf_emit(f, esc)){
     ret = -1;
@@ -510,8 +557,9 @@ ncplane* ncplane_new_internal(notcurses* nc, ncplane* n,
   if(p == NULL){
     return NULL;
   }
-  p->scrolling = false;
+  p->scrolling = nopts->flags & NCPLANE_OPTION_VSCROLL;
   p->fixedbound = nopts->flags & NCPLANE_OPTION_FIXED;
+  p->autogrow = nopts->flags & NCPLANE_OPTION_AUTOGROW;
   p->widget = NULL;
   p->wdestruct = NULL;
   if(nopts->flags & NCPLANE_OPTION_MARGINALIZED){
@@ -653,7 +701,7 @@ void ncplane_home(ncplane* n){
 int ncplane_cursor_move_yx(ncplane* n, int y, int x){
   if(x < 0){
     if(x < -1){
-      logerror("Negative target x %d\n", x);
+      logerror("negative target x %d\n", x);
       return -1;
     }
   }else if((unsigned)x >= n->lenx){
@@ -664,7 +712,7 @@ int ncplane_cursor_move_yx(ncplane* n, int y, int x){
   }
   if(y < 0){
     if(y < -1){
-      logerror("Negative target y %d\n", y);
+      logerror("negative target y %d\n", y);
       return -1;
     }
   }else if((unsigned)y >= n->leny){
@@ -769,17 +817,17 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx,
   }
   unsigned rows, cols;
   ncplane_dim_yx(n, &rows, &cols);
-  if(keepleny + keepy > (unsigned)rows){
+  if(keepleny + keepy > rows){
     logerror("can't keep %d@%d rows from %d\n", keepleny, keepy, rows);
     return -1;
   }
-  if(keeplenx + keepx > (unsigned)cols){
+  if(keeplenx + keepx > cols){
     logerror("can't keep %d@%d cols from %d\n", keeplenx, keepx, cols);
     return -1;
   }
   loginfo("%dx%d @ %d/%d → %u/%u @ %d/%d (want %ux%u@%d/%d)\n", rows, cols, n->absy, n->absx, ylen, xlen, n->absy + keepy + yoff, n->absx + keepx + xoff, keepleny, keeplenx, keepy, keepx);
   if(n->absy == n->absy + keepy && n->absx == n->absx + keepx &&
-      (unsigned)rows == ylen && (unsigned)cols == xlen){
+      rows == ylen && cols == xlen){
     return 0;
   }
   notcurses* nc = ncplane_notcurses(n);
@@ -793,30 +841,45 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx,
   int keptarea = keepleny * keeplenx;
   int newarea = ylen * xlen;
   size_t fbsize = sizeof(nccell) * newarea;
-  nccell* fb = malloc(fbsize);
-  if(fb == NULL){
-    return -1;
+  // an important optimization is the case where nothing needs to be moved,
+  // true when either the keptarea is 0 or the old and new x dimensions are
+  // equal, and we're keeping the full x dimension, and any material we're
+  // keeping starts at the top. in this case, we try to realloc() and avoid
+  // any copying whatsoever (we otherwise incur at least one copy due to
+  // always using a new area). set realloced high in this event, so we don't
+  // free anything.
+  nccell* fb;
+  bool realloced = false;
+  if(keptarea == 0 || (cols == xlen && cols == keeplenx && keepy == 0)){
+    if((fb = realloc(n->fb, fbsize)) == NULL){
+      return -1;
+    }
+    realloced = true;
+  }else{
+    if((fb = malloc(fbsize)) == NULL){
+      return -1;
+    }
   }
   if(n->tam){
     loginfo("TAM realloc to %d entries\n", newarea);
     // FIXME first, free any disposed auxiliary vectors!
     tament* tmptam = realloc(n->tam, sizeof(*tmptam) * newarea);
     if(tmptam == NULL){
-      free(fb);
+      if(!realloced){
+        free(fb);
+      }
       return -1;
     }
     n->tam = tmptam;
-    // FIXME need to set up the entries based on new distribution for
-    // cell-pixel geometry change, and split across new rows
     if(newarea > oldarea){
       memset(n->tam + oldarea, 0, sizeof(*n->tam) * (newarea - oldarea));
     }
   }
   // update the cursor, if it would otherwise be off-plane
-  if((unsigned)n->y >= ylen){
+  if(n->y >= ylen){
     n->y = ylen - 1;
   }
-  if((unsigned)n->x >= xlen){
+  if(n->x >= xlen){
     n->x = xlen - 1;
   }
   nccell* preserved = n->fb;
@@ -832,50 +895,57 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx,
   n->absy += keepy + yoff;
   n->absx += keepx + xoff;
 //fprintf(stderr, "absx: %d keepx: %d xoff: %d\n", n->absx, keepx, xoff);
-  if(keptarea == 0){ // keep nothing, resize/move only
+  if(keptarea == 0){ // keep nothing, resize/move only.
     // if we're keeping nothing, dump the old egcspool. otherwise, we go ahead
     // and keep it. perhaps we ought compact it?
     memset(fb, 0, sizeof(*fb) * newarea);
     egcpool_dump(&n->pool);
-    n->lenx = xlen;
-    n->leny = ylen;
-    free(preserved);
-    return resize_callbacks_children(n);
-  }
-  // we currently have maxy rows of maxx cells each. we will be keeping rows
-  // keepy..keepy + keepleny - 1 and columns keepx..keepx + keeplenx - 1.
-  // anything else is zerod out. itery is the row we're writing *to*, and we
-  // must write to each (and every cell in each).
-  for(unsigned itery = 0 ; itery < ylen ; ++itery){
-    int truey = itery + n->absy;
-    int sourceoffy = truey - oldabsy;
+  }else if(realloced){
+    // the x dimensions are equal, and we're keeping across the width. only the
+    // y dimension changed. at worst, we need zero some out.
+    unsigned tozorch = (ylen - keepleny) * xlen * sizeof(*fb);
+    if(tozorch){
+      unsigned zorchoff = keepleny * xlen;
+      memset(fb + zorchoff, 0, tozorch);
+    }
+  }else{
+    // we currently have maxy rows of maxx cells each. we will be keeping rows
+    // keepy..keepy + keepleny - 1 and columns keepx..keepx + keeplenx - 1.
+    // anything else is zerod out. itery is the row we're writing *to*, and we
+    // must write to each (and every cell in each).
+    for(unsigned itery = 0 ; itery < ylen ; ++itery){
+      int truey = itery + n->absy;
+      int sourceoffy = truey - oldabsy;
 //fprintf(stderr, "sourceoffy: %d keepy: %d ylen: %d\n", sourceoffy, keepy, ylen);
-    // if we have nothing copied to this line, zero it out in one go
-    if(sourceoffy < keepy || sourceoffy >= keepy + (int)keepleny){
+      // if we have nothing copied to this line, zero it out in one go
+      if(sourceoffy < keepy || sourceoffy >= keepy + (int)keepleny){
 //fprintf(stderr, "writing 0s to line %d of %d\n", itery, ylen);
-      memset(fb + (itery * xlen), 0, sizeof(*fb) * xlen);
-    }else{
-      int copyoff = itery * xlen; // our target at any given time
-      // we do have something to copy, and zero, one, or two regions to zero out
-      unsigned copied = 0;
-      if(xoff < 0){
-        memset(fb + copyoff, 0, sizeof(*fb) * -xoff);
-        copyoff += -xoff;
-        copied += -xoff;
-      }
-      const int sourceidx = nfbcellidx(n, sourceoffy, keepx);
+        memset(fb + (itery * xlen), 0, sizeof(*fb) * xlen);
+      }else{
+        int copyoff = itery * xlen; // our target at any given time
+        // we do have something to copy, and zero, one, or two regions to zero out
+        unsigned copied = 0;
+        if(xoff < 0){
+          memset(fb + copyoff, 0, sizeof(*fb) * -xoff);
+          copyoff += -xoff;
+          copied += -xoff;
+        }
+        const int sourceidx = nfbcellidx(n, sourceoffy, keepx);
 //fprintf(stderr, "copying line %d (%d) to %d (%d)\n", sourceoffy, sourceidx, copyoff / xlen, copyoff);
-      memcpy(fb + copyoff, preserved + sourceidx, sizeof(*fb) * keeplenx);
-      copyoff += keeplenx;
-      copied += keeplenx;
-      if(xlen > copied){
-        memset(fb + copyoff, 0, sizeof(*fb) * (xlen - copied));
+        memcpy(fb + copyoff, preserved + sourceidx, sizeof(*fb) * keeplenx);
+        copyoff += keeplenx;
+        copied += keeplenx;
+        if(xlen > copied){
+          memset(fb + copyoff, 0, sizeof(*fb) * (xlen - copied));
+        }
       }
     }
   }
   n->lenx = xlen;
   n->leny = ylen;
-  free(preserved);
+  if(!realloced){
+    free(preserved);
+  }
   return resize_callbacks_children(n);
 }
 
@@ -1042,6 +1112,10 @@ ncpixelimpl_e notcurses_check_pixel_support(const notcurses* nc){
 // iff we're using UTF8, |utf8| will be set to 1. it is otherwise set to 0.
 __attribute__ ((nonnull (2))) static notcurses*
 notcurses_early_init(const struct notcurses_options* opts, FILE* fp, unsigned* utf8){
+  if(fwide(fp, 0) > 0){
+    fprintf(stderr, "Error: output stream is wide-oriented\n");
+    return NULL;
+  }
   notcurses* ret = malloc(sizeof(*ret));
   if(ret == NULL){
     return ret;
@@ -1161,6 +1235,10 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
     free(ret);
     return NULL;
   }
+  if(ret->tcache.maxpaletteread > -1){
+    memcpy(ret->palette.chans, ret->tcache.originalpalette.chans,
+           sizeof(*ret->palette.chans) * (ret->tcache.maxpaletteread + 1));
+  }
   if((ret->flags & NCOPTION_PRESERVE_CURSOR) ||
       (!(ret->flags & NCOPTION_SUPPRESS_BANNERS))){
     // the u7 led the queries so that we would get a cursor position
@@ -1434,10 +1512,6 @@ const char* nccell_extended_gcluster(const ncplane* n, const nccell* c){
   return egcpool_extended_gcluster(&n->pool, c);
 }
 
-const char* cell_extended_gcluster(const struct ncplane* n, const nccell* c){
-  return nccell_extended_gcluster(n, c);
-}
-
 // 'n' ends up above 'above'
 int ncplane_move_above(ncplane* restrict n, ncplane* restrict above){
   if(n == above){ // probably gets optimized out =/
@@ -1620,6 +1694,11 @@ void scroll_down(ncplane* n){
 //fprintf(stderr, "pre-scroll: %d/%d %d/%d log: %d scrolling: %u\n", n->y, n->x, n->leny, n->lenx, n->logrow, n->scrolling);
   n->x = 0;
   if(n->y == n->leny - 1){
+    if(n->autogrow){
+      ncplane_resize_simple(n, n->leny + 1, n->lenx);
+      ncplane_cursor_move_yx(n, n->leny - 1, 0);
+      return;
+    }
     if(n == notcurses_stdplane(ncplane_notcurses(n))){
       ncplane_pile(n)->scrolls++;
     }
@@ -1685,10 +1764,6 @@ int nccell_load(ncplane* n, nccell* c, const char* gcluster){
   return pool_load_direct(&n->pool, c, gcluster, bytes, cols);
 }
 
-int cell_load(ncplane* n, nccell* c, const char* gcluster){
-  return nccell_load(n, c, gcluster);
-}
-
 // where the magic happens. write the single EGC completely described by |egc|,
 // occupying |cols| columns, to the ncplane |n| at the coordinate |y|, |x|. if
 // either or both of |y|/|x| is -1, the current cursor location for that
@@ -1698,36 +1773,51 @@ static int
 ncplane_put(ncplane* n, int y, int x, const char* egc, int cols,
             uint16_t stylemask, uint64_t channels, int bytes){
   if(n->sprite){
-    logerror("Can't write [%s] to sprixelated plane\n", egc);
+    logerror("can't write [%s] to sprixelated plane\n", egc);
     return -1;
   }
   // reject any control character for output other than newline (and then only
   // on a scrolling plane).
   if(*egc == '\n'){
+    // if we're not scrolling, autogrow would be to the right (as opposed to
+    // down), and thus it still wouldn't apply to the case of a newline.
     if(!n->scrolling){
-      logerror("Rejecting newline on non-scrolling plane\n");
+      logerror("rejecting newline on non-scrolling plane\n");
       return -1;
     }
   }else if(is_control_egc((const unsigned char*)egc, bytes)){
-    logerror("Rejecting %dB control character\n", bytes);
+    logerror("rejecting %dB control character\n", bytes);
     return -1;
   }
   // check *before ncplane_cursor_move_yx()* whether we're past the end of the
   // line. if scrolling is enabled, move to the next line if so. if x or y are
-  // specified, we must always try to print at exactly that location.
-  if(x != -1){
-    if((unsigned)x + cols > n->lenx){
-      logerror("Target x %d + %d cols [%.*s] > length %d\n", x, cols, bytes, egc, n->lenx);
-      ncplane_cursor_move_yx(n, y, x); // update cursor, though
-      return -1;
+  // specified, we must always try to print at exactly that location, and
+  // there's no need to check the present location in that dimension.
+  bool linesend = false;
+  if(x < 0){
+    // we checked x for all negatives, but only -1 is valid (our else clause is
+    // predicated on a non-negative x).
+    if(x == -1){
+      if(n->x + cols - 1 >= n->lenx){
+        linesend = true;
+      }
     }
-  }else if(y == -1 && n->x + cols > n->lenx){
-    if(!n->scrolling){
-      logerror("No room to output [%.*s] %d/%d\n", bytes, egc, n->y, n->x);
-      return -1;
+  }else{
+    if((unsigned)x + cols - 1 >= n->lenx){
+      linesend = true;
     }
-    scroll_down(n);
   }
+  if(linesend){
+    if(n->scrolling){
+      scroll_down(n);
+    }else if(n->autogrow){
+      ncplane_resize_simple(n, n->leny, n->lenx + cols);
+    }else{
+      logerror("target x %d [%.*s] > length %d\n", n->x, bytes, egc, n->lenx);
+      return -1;
+    }
+  }
+  // targets outside the plane will be rejected here.
   if(ncplane_cursor_move_yx(n, y, x)){
     return -1;
   }
@@ -1782,8 +1872,17 @@ ncplane_put(ncplane* n, int y, int x, const char* egc, int cols,
 
 int ncplane_putc_yx(ncplane* n, int y, int x, const nccell* c){
   const int cols = nccell_cols(c);
-  const char* egc = nccell_extended_gcluster(n, c);
-  return ncplane_put(n, y, x, egc, cols, c->stylemask, c->channels, strlen(egc));
+  // unfortunately, |c| comes from |n|. the act of writing |c| to |n| could
+  // grow |n|'s egcpool, invalidating the reference represented by
+  // nccell_extended_gcluster(). so we must copy and free it.
+  char* egc = nccell_strdup(n, c);
+  if(egc == NULL){
+    logerror("coudln't duplicate cell\n");
+    return -1;
+  }
+  int r = ncplane_put(n, y, x, egc, cols, c->stylemask, c->channels, strlen(egc));
+  free(egc);
+  return r;
 }
 
 int ncplane_putegc_yx(ncplane* n, int y, int x, const char* gclust, size_t* sbytes){
@@ -2118,7 +2217,7 @@ int ncplane_box(ncplane* n, const nccell* ul, const nccell* ur,
   }
   ++yoff;
   // middle rows (vertical lines)
-  if((unsigned)yoff < ystop){
+  if(yoff < ystop){
     if(!(ctlword & NCBOXMASK_LEFT)){
       if(ncplane_cursor_move_yx(n, yoff, xoff)){
         return -1;
@@ -2262,7 +2361,7 @@ void ncplane_erase(ncplane* n){
   memset(n->fb, 0, sizeof(*n->fb) * n->leny * n->lenx);
   egcpool_dump(&n->pool);
   egcpool_init(&n->pool);
-  // we need to zero out the EGC before handing this off to cell_load, but
+  // we need to zero out the EGC before handing this off to nccell_load, but
   // we don't want to lose the channels/attributes, so explicit gcluster load.
   n->basecell.gcluster = 0;
   nccell_load(n, &n->basecell, egc);
@@ -2786,7 +2885,7 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
   return n;
 }
 
-bool ncplane_set_scrolling(ncplane* n, bool scrollp){
+bool ncplane_set_scrolling(ncplane* n, unsigned scrollp){
   bool old = n->scrolling;
   n->scrolling = scrollp;
   return old;
@@ -2796,6 +2895,20 @@ bool ncplane_scrolling_p(const ncplane* n){
   return n->scrolling;
 }
 
+bool ncplane_set_autogrow(ncplane* n, unsigned growp){
+  if(n == notcurses_stdplane_const(ncplane_notcurses_const(n))){
+    logerror("can't set the standard plane autogrow\n");
+    return false;
+  }
+  bool old = n->autogrow;
+  n->autogrow = growp;
+  return old;
+}
+
+bool ncplane_autogrow_p(const ncplane* n){
+  return n->autogrow;
+}
+
 // extract an integer, which must be non-negative, and followed by either a
 // comma or a NUL terminator.
 static int
