@@ -3,6 +3,7 @@
 #include <unistd.h>
 #include <notcurses/direct.h>
 #include "internal.h"
+#include "unixsig.h"
 
 sig_atomic_t sigcont_seen_for_render = 0;
 
@@ -298,37 +299,25 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
       if(nccell_wide_right_p(targc)){
         continue;
       }
-      // if we encounter a newline (always a bare linefeed; Windows CRLF
-      // translation only matters for text stdio), we're done with this
-      // line *for this plane*.
-      if(targc->gcluster == '\n'){
-        break;
-      }
 
       if(nccell_fg_alpha(targc) > NCALPHA_OPAQUE){
         const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
         if(nccell_fg_default_p(vis)){
           vis = &p->basecell;
         }
-        if(nccell_fg_palindex_p(vis)){
-          if(nccell_fg_alpha(targc) == NCALPHA_TRANSPARENT){
-            nccell_set_fg_palindex(targc, nccell_fg_palindex(vis));
-          }
-        }else{
-          if(nccell_fg_alpha(vis) == NCALPHA_HIGHCONTRAST){
-            crender->s.highcontrast = true;
-            crender->s.hcfgblends = crender->s.fgblends;
-            crender->hcfg = cell_fchannel(targc);
-          }
-          unsigned fgblends = crender->s.fgblends;
-          cell_blend_fchannel(targc, cell_fchannel(vis), &fgblends);
-          crender->s.fgblends = fgblends;
-          // crender->highcontrast can only be true if we just set it, since we're
-          // about to set targc opaque based on crender->highcontrast (and this
-          // entire stanza is conditional on targc not being NCALPHA_OPAQUE).
-          if(crender->s.highcontrast){
-            nccell_set_fg_alpha(targc, NCALPHA_OPAQUE);
-          }
+        if(nccell_fg_alpha(vis) == NCALPHA_HIGHCONTRAST){
+          crender->s.highcontrast = true;
+          crender->s.hcfgblends = crender->s.fgblends;
+          crender->hcfg = cell_fchannel(targc);
+        }
+        unsigned fgblends = crender->s.fgblends;
+        cell_blend_fchannel(ncplane_notcurses(p), targc, cell_fchannel(vis), &fgblends);
+        crender->s.fgblends = fgblends;
+        // crender->highcontrast can only be true if we just set it, since we're
+        // about to set targc opaque based on crender->highcontrast (and this
+        // entire stanza is conditional on targc not being NCALPHA_OPAQUE).
+        if(crender->s.highcontrast){
+          nccell_set_fg_alpha(targc, NCALPHA_OPAQUE);
         }
       }
 
@@ -347,28 +336,16 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
           if(nccell_bg_default_p(vis)){
             vis = &p->basecell;
           }
-          if(nccell_bg_palindex_p(vis)){
-            if(nccell_bg_alpha(targc) == NCALPHA_TRANSPARENT){
-              nccell_set_bg_palindex(targc, nccell_bg_palindex(vis));
-            }
-          }else{
-            unsigned bgblends = crender->s.bgblends;
-            cell_blend_bchannel(targc, cell_bchannel(vis), &bgblends);
-            crender->s.bgblends = bgblends;
-          }
+          unsigned bgblends = crender->s.bgblends;
+          cell_blend_bchannel(ncplane_notcurses(p), targc, cell_bchannel(vis), &bgblends);
+          crender->s.bgblends = bgblends;
         }else{ // use the local foreground; we're stacking blittings
           if(nccell_fg_default_p(vis)){
             vis = &p->basecell;
           }
-          if(nccell_fg_palindex_p(vis)){
-            if(nccell_bg_alpha(targc) == NCALPHA_TRANSPARENT){
-              nccell_set_bg_palindex(targc, nccell_fg_palindex(vis));
-            }
-          }else{
-            unsigned bgblends = crender->s.bgblends;
-            cell_blend_bchannel(targc, cell_fchannel(vis), &bgblends);
-            crender->s.bgblends = bgblends;
-          }
+          unsigned bgblends = crender->s.bgblends;
+          cell_blend_bchannel(ncplane_notcurses(p), targc, cell_fchannel(vis), &bgblends);
+          crender->s.bgblends = bgblends;
           crender->s.blittedquads = 0;
         }
       }
@@ -439,7 +416,7 @@ init_rvec(struct crender* rvec, int totalcells){
 // should be done at the end of rendering the cell, so that contrast is solved
 // against the real background.
 static inline void
-lock_in_highcontrast(const tinfo* ti, nccell* targc, struct crender* crender){
+lock_in_highcontrast(notcurses* nc, const tinfo* ti, nccell* targc, struct crender* crender){
   if(nccell_fg_alpha(targc) == NCALPHA_TRANSPARENT){
     nccell_set_fg_default(targc);
   }
@@ -452,10 +429,10 @@ lock_in_highcontrast(const tinfo* ti, nccell* targc, struct crender* crender){
       unsigned fgblends = 3;
       uint32_t fchan = cell_fchannel(targc);
       uint32_t bchan = cell_bchannel(targc);
-      uint32_t hchan = channels_blend(highcontrast(ti, bchan), fchan, &fgblends);
+      uint32_t hchan = channels_blend(nc, highcontrast(ti, bchan), fchan, &fgblends);
       cell_set_fchannel(targc, hchan);
       fgblends = crender->s.hcfgblends;
-      hchan = channels_blend(hchan, crender->hcfg, &fgblends);
+      hchan = channels_blend(nc, hchan, crender->hcfg, &fgblends);
       cell_set_fchannel(targc, hchan);
     }else{
       nccell_set_fg_rgb(targc, highcontrast(ti, cell_bchannel(targc)));
@@ -467,10 +444,10 @@ lock_in_highcontrast(const tinfo* ti, nccell* targc, struct crender* crender){
 // checking for and locking in high-contrast, checking for damage, and updating
 // 'lastframe' for any cells which are damaged.
 static inline void
-postpaint_cell(const tinfo* ti, nccell* lastframe, int dimx,
+postpaint_cell(notcurses* nc, const tinfo* ti, nccell* lastframe, int dimx,
                struct crender* crender, egcpool* pool, int y, int* x){
   nccell* targc = &crender->c;
-  lock_in_highcontrast(ti, targc, crender);
+  lock_in_highcontrast(nc, ti, targc, crender);
   nccell* prevcell = &lastframe[fbcellidx(y, dimx, *x)];
   if(cellcmp_and_dupfar(pool, prevcell, crender->p, targc) > 0){
 //fprintf(stderr, "damaging due to cmp [%s] %d %d\n", nccell_extended_gcluster(crender->p, &crender->c), y, *x);
@@ -520,12 +497,12 @@ postpaint_cell(const tinfo* ti, nccell* lastframe, int dimx,
 // FIXME can we not do the blend a single time here, if we track sums in
 //       paint()? tried this before and didn't get a win...
 static void
-postpaint(const tinfo* ti, nccell* lastframe, int dimy, int dimx,
+postpaint(notcurses* nc, const tinfo* ti, nccell* lastframe, int dimy, int dimx,
           struct crender* rvec, egcpool* pool){
   for(int y = 0 ; y < dimy ; ++y){
     for(int x = 0 ; x < dimx ; ++x){
       struct crender* crender = &rvec[fbcellidx(y, dimx, x)];
-      postpaint_cell(ti, lastframe, dimx, crender, pool, y, &x);
+      postpaint_cell(nc, ti, lastframe, dimx, crender, pool, y, &x);
     }
   }
 }
@@ -618,7 +595,7 @@ int ncplane_mergedown(ncplane* restrict src, ncplane* restrict dst,
   assert(NULL == s);
 //fprintf(stderr, "Postpaint start (%dx%d)\n", dst->leny, dst->lenx);
   const struct tinfo* ti = &ncplane_notcurses_const(dst)->tcache;
-  postpaint(ti, rendfb, dst->leny, dst->lenx, rvec, &dst->pool);
+  postpaint(ncplane_notcurses(dst), ti, rendfb, dst->leny, dst->lenx, rvec, &dst->pool);
 //fprintf(stderr, "Postpaint done (%dx%d)\n", dst->leny, dst->lenx);
   free(dst->fb);
   dst->fb = rendfb;
@@ -793,6 +770,7 @@ update_palette(notcurses* nc, fbuf* f){
     for(size_t damageidx = 0 ; damageidx < sizeof(nc->palette.chans) / sizeof(*nc->palette.chans) ; ++damageidx){
       unsigned r, g, b;
       if(nc->palette_damage[damageidx]){
+        nc->touched_palette = true;
         ncchannel_rgb8(nc->palette.chans[damageidx], &r, &g, &b);
         // Need convert RGB values [0..256) to [0..1000], ugh
         // FIXME need handle HSL case also
@@ -1544,7 +1522,7 @@ int ncpile_rasterize(ncplane* n){
   const int miny = pile->dimy < nc->lfdimy ? pile->dimy : nc->lfdimy;
   const int minx = pile->dimx < nc->lfdimx ? pile->dimx : nc->lfdimx;
   const struct tinfo* ti = &ncplane_notcurses_const(n)->tcache;
-  postpaint(ti, nc->lastframe, miny, minx, pile->crender, &nc->pool);
+  postpaint(nc, ti, nc->lastframe, miny, minx, pile->crender, &nc->pool);
   clock_gettime(CLOCK_MONOTONIC, &rasterdone);
   int bytes = notcurses_rasterize(nc, pile, &nc->rstate.f);
   // accepts -1 as an indication of failure
@@ -1725,7 +1703,27 @@ int ncdirect_set_fg_rgb(ncdirect* nc, unsigned rgb){
   return 0;
 }
 
-int notcurses_cursor_yx(notcurses* nc, int* y, int* x){
+int notcurses_default_foreground(const struct notcurses* nc, uint32_t* fg){
+  const tinfo* ti = &nc->tcache;
+  if(ti->fg_default & 0x80000000){
+    logerror("default foreground could not be determined\n");
+    return -1;
+  }
+  *fg = ti->fg_default & NC_BG_RGB_MASK;
+  return 0;
+}
+
+int notcurses_default_background(const struct notcurses* nc, uint32_t* bg){
+  const tinfo* ti = &nc->tcache;
+  if(ti->bg_collides_default & 0x80000000){
+    logerror("default background could not be determined\n");
+    return -1;
+  }
+  *bg = ti->bg_collides_default & NC_BG_RGB_MASK;
+  return 0;
+}
+
+int notcurses_cursor_yx(const notcurses* nc, int* y, int* x){
   *y = nc->rstate.y;
   *x = nc->rstate.x;
   return 0;
