@@ -16,8 +16,13 @@ extern "C" {
 #include "fbuf.h"
 #include "in.h"
 
+// kitty keyboard protocol pop, used at end when kitty is verified.
 #define KKEYBOARD_POP  "\x1b[<u"
-#define KKEYBOARD_PUSH "\x1b[>u"
+
+// disable key modifier options; this corresponds to a resource value of
+// "-1", which cannot be set with the [>m sequence. supposedly, "[>m" by
+// itself ought reset all of them, but this doesn't seem to work FIXME.
+#define XTMODKEYSUNDO "\x1b[>2n\x1b[>4n"
 
 struct ncpile;
 struct sprixel;
@@ -105,9 +110,9 @@ typedef struct tinfo {
   unsigned pixx;                   // total pixel geometry, width
   // we use the cell's size in pixels for pixel blitting. this information can
   // be acquired on all terminals with pixel support.
-  unsigned cellpixy;               // cell pixel height, might be 0
-  unsigned cellpixx;               // cell pixel width, might be 0
-  int dimy, dimx;                  // most recent cell geometry
+  unsigned cellpxy;                // cell pixel height, might be 0
+  unsigned cellpxx;                // cell pixel width, might be 0
+  unsigned dimy, dimx;             // most recent cell geometry
 
   unsigned supported_styles; // bitmask over NCSTYLE_* driven via sgr/ncv
 
@@ -118,6 +123,9 @@ typedef struct tinfo {
   // 0xfexxxxxxx (unknown), 0x00RRGGBB (no collide), or 0x01RRGGBB (collides).
   uint32_t bg_collides_default;
 
+  // 0xffxxxxxxx (unknown), or 0x00RRGGBB (foreground)
+  uint32_t fg_default;
+
   // bitmap support. if we support bitmaps, pixel_implementation will be a
   // value other than NCPIXEL_NONE.
   ncpixelimpl_e pixel_implementation;
@@ -144,22 +152,22 @@ typedef struct tinfo {
   int (*pixel_commit)(fbuf* f, struct sprixel* s, unsigned noscroll);
   // scroll all graphics up. only used with fbcon.
   void (*pixel_scroll)(const struct ncpile* p, struct tinfo*, int rows);
-  uint8_t* (*pixel_trans_auxvec)(const struct tinfo* ti); // create tranparent auxvec
+  uint8_t* (*pixel_trans_auxvec)(const struct ncpile* p); // create tranparent auxvec
   // sprixel parameters. there are several different sprixel protocols, of
   // which we support sixel and kitty. the kitty protocol is used based
   // on TERM heuristics. otherwise, we attempt to detect sixel support, and
   // query the details of the implementation.
   int color_registers; // sixel color registers (post pixel_query_done)
-  int sixel_maxx;      // maximum theoretical sixel width
+  unsigned sixel_maxx; // maximum theoretical sixel width
   // in sixel, we can't render to the bottom row, lest we force a one-line
   // scroll. we thus clamp sixel_maxy_pristine to the minimum of
   // sixel_maxy_pristine (the reported sixel_maxy), and the number of rows
   // less one times the cell height. sixel_maxy is thus recomputed whenever
   // we get a resize event. it is only defined if we have sixel_maxy_pristine,
   // so kitty graphics (which don't force a scroll) never deal with this.
-  int sixel_maxy;            // maximum working sixel height
-  int sixel_maxy_pristine;   // maximum theoretical sixel height, as queried
-  int sprixel_scale_height;  // sprixel must be a multiple of this many rows
+  unsigned sixel_maxy;          // maximum working sixel height
+  unsigned sixel_maxy_pristine; // maximum theoretical sixel height, as queried
+  unsigned sprixel_scale_height;// sprixel must be a multiple of this many rows
   const char* termname;      // terminal name from environment variables/init
   char* termversion;         // terminal version (freeform) from query responses
   queried_terminals_e qterm; // detected terminal class
@@ -167,6 +175,8 @@ typedef struct tinfo {
   struct termios *tpreserved;// terminal state upon entry
   struct inputctx* ictx;     // new input layer
   unsigned stdio_blocking_save; // was stdio blocking at entry? restore on stop.
+  // ought we issue gratuitous HPAs to work around ambiguous widths?
+  unsigned gratuitous_hpa;
 
   // if we get a reply to our initial \e[18t cell geometry query, it will
   // replace these values. note that LINES/COLUMNS cannot be used to limit
@@ -174,8 +184,11 @@ typedef struct tinfo {
   int default_rows;          // LINES environment var / lines terminfo / 24
   int default_cols;          // COLUMNS environment var / cols terminfo / 80
 
-  int gpmfd;                 // connection to GPM daemon
+  ncpalette originalpalette; // palette as read from initial queries
+  int maxpaletteread;        // maximum palette entry read
   pthread_t gpmthread;       // thread handle for GPM watcher
+  int gpmfd;                 // connection to GPM daemon
+  char mouseproto;           // DECSET level (100x, '0', '2', '3')
 #ifdef __linux__
   int linux_fb_fd;           // linux framebuffer device fd
   char* linux_fb_dev;        // device corresponding to linux_fb_dev
@@ -191,19 +204,15 @@ typedef struct tinfo {
   // pop the keyboard support level, which we normally do only if we detected
   // actual support. at that point, we obviously haven't detected anything).
   // after getting the initialization package back, if it's still UINT_MAX, we
-  // set it back to 0, since we know at that point that there's no support.
+  // set it to 0, and also indicate a lack of support via kittykbdsupport (we
+  // need distinguish between level 0, used with DRAININPUT, and an absolute
+  // lack of support, in which case we move to XTMODKEYS, for notcurses-info).
   unsigned kbdlevel;         // kitty keyboard support level
+  bool kittykbdsupport;      // do we support the kitty keyboard protocol?
   bool bce;                  // is the bce property advertised?
   bool in_alt_screen;        // are we in the alternate screen?
 } tinfo;
 
-static inline void
-update_tinfo_geometry(tinfo* ti, int dimy, int dimx){
-  // FIXME probably need a lock on this!
-  ti->dimy = dimy;
-  ti->dimx = dimx;
-}
-
 // retrieve the terminfo(5)-style escape 'e' from tdesc (NULL if undefined).
 static inline __attribute__ ((pure)) const char*
 get_escape(const tinfo* tdesc, escape_e e){
@@ -214,7 +223,7 @@ get_escape(const tinfo* tdesc, escape_e e){
   return NULL;
 }
 
-static inline int
+static inline uint16_t
 term_supported_styles(const tinfo* ti){
   return ti->supported_styles;
 }
@@ -235,7 +244,7 @@ void free_terminfo_cache(tinfo* ti);
 // return a heap-allocated copy of termname + termversion
 char* termdesc_longterm(const tinfo* ti);
 
-int locate_cursor(tinfo* ti, int* cursor_y, int* cursor_x);
+int locate_cursor(tinfo* ti, unsigned* cursor_y, unsigned* cursor_x);
 
 // tlen -- size of escape table. tused -- used bytes in same.
 // returns -1 if the starting location is >= 65535. otherwise,
@@ -325,8 +334,9 @@ term_emit(const char* seq, FILE* out, bool flush){
   return flush ? ncflush(out) : 0;
 }
 
-int enter_alternate_screen(FILE* fp, tinfo* ti, bool flush);
-int leave_alternate_screen(FILE* fp, tinfo* ti);
+// |drain| is set iff we're draining input.
+int enter_alternate_screen(int ttyfd, FILE* ttyfp, tinfo* ti, unsigned drain);
+int leave_alternate_screen(int ttyfd, FILE* ttyfp, tinfo* ti, unsigned drain);
 
 int cbreak_mode(tinfo* ti);
 
