@@ -14,7 +14,7 @@ typedef struct nctree {
   nctree_int_item* curitem; // item addressed by the path
   unsigned maxdepth;        // maximum hierarchy level
   unsigned* currentpath;    // array of |maxdepth|+1 elements, ended by UINT_MAX
-  int activerow;            // active row 0 <= activerow < dimy
+  int activerow;            // active row -1 <= activerow < dimy
   int indentcols;           // cols to indent per level
   uint64_t bchannels;       // border glyph channels
 } nctree;
@@ -33,8 +33,8 @@ free_tree_items(nctree_int_item* iarray){
 // allocates a |count|-sized array of nctree_int_items, and fills |fill| in,
 // using |items|. updates |*maxdepth| when appropriate.
 static int
-dup_tree_items(nctree_int_item* fill, const nctree_item* items, unsigned count, unsigned depth,
-               unsigned* maxdepth){
+dup_tree_items(nctree_int_item* fill, const struct nctree_item* items,
+               unsigned count, unsigned depth, unsigned* maxdepth){
   fill->subcount = count;
   // FIXME perhaps better to alloc a page at a time and take all items from
   // there, for better TLB performance?
@@ -81,10 +81,16 @@ goto_last_item(nctree* n){
 
 static void
 goto_first_item(nctree* n){
-  n->currentpath[0] = 0;
-  n->currentpath[1] = UINT_MAX;
-  n->curitem = &n->items.subs[0];
-  n->activerow = 0;
+  if(n->maxdepth == 0){
+    n->currentpath[0] = UINT_MAX;
+    n->curitem = NULL;
+    n->activerow = -1;
+  }else{
+    n->currentpath[0] = 0;
+    n->currentpath[1] = UINT_MAX;
+    n->curitem = &n->items.subs[0];
+    n->activerow = 0;
+  }
 }
 
 // the initial path ought point to the first item.
@@ -122,12 +128,97 @@ nctree_inner_create(ncplane* n, const nctree_options* opts){
   return ret;
 }
 
+int nctree_add(nctree* n, const unsigned* spec, const struct nctree_item* add){
+  // it's illegal to pass an empty path for addition; one must pass { 0, UINT_MAX }
+  if(spec[0] == UINT_MAX){
+    logerror("invalid empty path\n");
+    return -1;
+  }
+  struct nctree_int_item* nii = &n->items;
+  const unsigned* p = spec;
+  unsigned depth = 0;
+  while(p[1] != UINT_MAX){ // we know p[1] isn't UINT_MAX
+    if(*p >= nii->subcount){
+      logerror("invalid path element (%u >= %u)\n", *p, nii->subcount);
+      return -1;
+    }
+    nii = &nii->subs[*p];
+    ++p;
+    ++depth;
+  }
+  // this last one can be equal to subcount; we're placing it at the end
+  if(*p > nii->subcount){
+    logerror("invalid path element (%u >= %u)\n", *p, nii->subcount);
+    return -1;
+  }
+  struct nctree_int_item* tmparr = realloc(nii->subs, sizeof(*nii->subs) * (nii->subcount + 1));
+  if(tmparr == NULL){
+    return -1;
+  }
+  nii->subs = tmparr;
+  if(*p != nii->subcount++){
+    memmove(&nii->subs[*p], &nii->subs[*p + 1], sizeof(*nii->subs) * (nii->subcount - *p));
+  }
+  if(p - spec >= n->maxdepth){
+    unsigned max = p - spec + 1;
+    unsigned* tmp = realloc(n->currentpath, sizeof(*n->currentpath) * (max + 1));
+    if(tmp == NULL){
+      return -1;
+    }
+    n->currentpath = tmp;
+    n->currentpath[max] = UINT_MAX;
+    n->maxdepth = max;
+  }
+  nii->subs[*p].subs = NULL;
+  nii->subs[*p].subcount = 0;
+  nii->subs[*p].curry = NULL;
+  nii->subs[*p].ncp = NULL;
+  if(dup_tree_items(&nii->subs[*p], add->subs, add->subcount, depth, &n->maxdepth)){
+    return -1;
+  }
+  if(n->activerow == -1){
+    n->activerow = 0;
+    n->curitem = &n->items;
+    n->currentpath[0] = 0;
+  }
+  return 0;
+}
+
+int nctree_del(nctree* n, const unsigned* spec){
+  nctree_int_item* parent = NULL;
+  nctree_int_item* nii = &n->items;
+  const unsigned* p = spec;
+  while(*p != UINT_MAX){
+    if(*p >= nii->subcount){
+      logerror("invalid path element (%u >= %u)\n", *p, nii->subcount);
+      return -1;
+    }
+    parent = nii;
+    nii = &nii->subs[*p];
+    ++p;
+  }
+  free_tree_items(nii);
+  if(parent){
+    // parent can only be defined if we had at least one path element
+    unsigned lastelem = spec[-1];
+    if(lastelem != --parent->subcount){
+      memmove(&parent->subs[lastelem], &parent->subs[lastelem + 1],
+              sizeof(*parent->subs) * (parent->subcount - lastelem));
+    }
+  }
+  if(n->items.subcount == 0){
+    n->activerow = -1;
+    n->curitem = NULL;
+  }
+  return 0;
+}
+
 nctree* nctree_create(ncplane* n, const nctree_options* opts){
   if(opts->flags){
     logwarn("Passed invalid flags 0x%016" PRIx64 "\n", opts->flags);
   }
-  if(opts->count == 0 || opts->items == NULL){
-    logerror("Can't create empty tree\n");
+  if(n == notcurses_stdplane(ncplane_notcurses(n))){
+    logerror("can't use the standard plane\n");
     goto error;
   }
   if(opts->nctreecb == NULL){
@@ -153,6 +244,7 @@ error:
 void nctree_destroy(nctree* n){
   if(n){
     free_tree_items(&n->items);
+    free(n->currentpath);
     free(n);
   }
 }
@@ -260,7 +352,7 @@ void* nctree_next(nctree* n){
   if(tmp != n->curitem){
     n->curitem = tmp;
     n->activerow += rows;
-    if(n->activerow >= ncplane_dim_y(n->items.ncp)){
+    if(n->activerow >= (int)ncplane_dim_y(n->items.ncp)){
       n->activerow = ncplane_dim_y(n->items.ncp) - 1;
     }
   }
@@ -282,7 +374,6 @@ tree_path_length(const unsigned* path){
 static int
 draw_tree_item(nctree* n, nctree_int_item* nii, const unsigned* path,
                int* frontiert, int* frontierb, int distance){
-//fprintf(stderr, "drawing item ft: %d fb: %d %p\n", *frontiert, *frontierb, nii->ncp);
   if(!nii->ncp){
     const int startx = (tree_path_length(path) - 1) * n->indentcols;
     int ymin, ymax;
@@ -314,7 +405,7 @@ draw_tree_item(nctree* n, nctree_int_item* nii, const unsigned* path,
   }else{
     // FIXME possibly enlarge nii->ncp?
   }
-  if(ncplane_y(nii->ncp) <= *frontiert || *frontierb >= ncplane_dim_y(n->items.ncp)){
+  if(ncplane_y(nii->ncp) <= *frontiert || *frontierb >= (int)ncplane_dim_y(n->items.ncp)){
     ncplane_move_yx(nii->ncp, *frontiert, ncplane_x(nii->ncp));
   }else{
     ncplane_move_yx(nii->ncp, *frontierb, ncplane_x(nii->ncp));
@@ -328,7 +419,7 @@ draw_tree_item(nctree* n, nctree_int_item* nii, const unsigned* path,
   if(ncplane_y(nii->ncp) <= *frontiert){
     *frontiert = ncplane_y(nii->ncp) - 1;
   }
-  if(ncplane_y(nii->ncp) + ncplane_dim_y(nii->ncp) > *frontierb){
+  if(ncplane_y(nii->ncp) + (int)ncplane_dim_y(nii->ncp) > *frontierb){
     *frontierb = ncplane_y(nii->ncp) + ncplane_dim_y(nii->ncp);
   }
   return 0;
@@ -374,6 +465,9 @@ destroy_below(nctree* n, nctree_int_item* nii, unsigned* path, int distance){
 // to hold n->maxdepth + 1 unsigneds.
 static int
 nctree_inner_redraw(nctree* n, unsigned* tmppath){
+  if(n->activerow < 0){
+    return 0;
+  }
   ncplane* ncp = n->items.ncp;
   if(ncplane_cursor_move_yx(ncp, n->activerow, 0)){
     return -1;
@@ -407,7 +501,7 @@ nctree_inner_redraw(nctree* n, unsigned* tmppath){
   nii = n->curitem;
   // draw items below the current one FIXME
   memcpy(tmppath, n->currentpath, sizeof(*tmppath) * (n->maxdepth + 1));
-  while(frontierb < ncplane_dim_y(n->items.ncp)){
+  while(frontierb < (int)ncplane_dim_y(n->items.ncp)){
     if((tmpnii = nctree_next_internal(n, tmppath)) == nii){
       break;
     }
