@@ -22,13 +22,11 @@ struct AVCodecParameters;
 struct AVPacket;
 
 typedef struct ncvisual_details {
-  int packet_outstanding;
   struct AVFormatContext* fmtctx;
   struct AVCodecContext* codecctx;     // video codec context
   struct AVCodecContext* subtcodecctx; // subtitle codec context
-  struct AVFrame* frame;               // frame as read
+  struct AVFrame* frame;               // frame as read/loaded/converted
   struct AVCodec* codec;
-  struct AVCodecParameters* cparams;
   struct AVCodec* subtcodec;
   struct AVPacket* packet;
   struct SwsContext* swsctx;
@@ -36,6 +34,7 @@ typedef struct ncvisual_details {
   AVSubtitle subtitle;
   int stream_index;        // match against this following av_read_frame()
   int sub_stream_index;    // subtitle stream index, can be < 0 if no subtitles
+  bool packet_outstanding;
 } ncvisual_details;
 
 #define IMGALLOCALIGN 64
@@ -74,8 +73,8 @@ print_frame_summary(const AVCodecContext* cctx, const AVFrame* f){
   if(f->palette_has_changed){
     fprintf(stderr, " [NewPal]");
   }
-  fprintf(stderr, " PTS %ld Flags: 0x%04x\n", f->pts, f->flags);
-  fprintf(stderr, " %lums@%lums (%skeyframe) qual: %d\n",
+  fprintf(stderr, " PTS %" PRId64 " Flags: 0x%04x\n", f->pts, f->flags);
+  fprintf(stderr, " %" PRIu64 "ms@%" PRIu64 "ms (%skeyframe) qual: %d\n",
           f->pkt_duration, // FIXME in 'time_base' units
           f->best_effort_timestamp,
           f->key_frame ? "" : "non-",
@@ -121,7 +120,7 @@ subtitle_plane_from_text(ncplane* parent, const char* text){
 //logerror("need a parent plane\n");
     return NULL;
   }
-  int width = ncstrwidth(text);
+  int width = ncstrwidth(text, NULL, NULL);
   if(width <= 0){
 //logwarn("couldn't extract subtitle from %s\n", text);
     return NULL;
@@ -141,7 +140,7 @@ subtitle_plane_from_text(ncplane* parent, const char* text){
   uint64_t channels = 0;
   ncchannels_set_fg_alpha(&channels, NCALPHA_HIGHCONTRAST);
   ncchannels_set_fg_rgb8(&channels, 0x88, 0x88, 0x88);
-  ncplane_stain(n, -1, -1, channels, channels, channels, channels);
+  ncplane_stain(n, -1, -1, 0, 0, channels, channels, channels, channels);
   ncchannels_set_fg_default(&channels);
   ncplane_puttext(n, 0, NCALIGN_LEFT, text, NULL);
   ncchannels_set_bg_alpha(&channels, NCALPHA_TRANSPARENT);
@@ -149,7 +148,7 @@ subtitle_plane_from_text(ncplane* parent, const char* text){
   return n;
 }
 
-static uint32_t palette[256];
+static uint32_t palette[NCPALETTESIZE];
 
 struct ncplane* ffmpeg_subtitle(ncplane* parent, const ncvisual* ncv){
   for(unsigned i = 0 ; i < ncv->details->subtitle.num_rects ; ++i){
@@ -174,19 +173,21 @@ struct ncplane* ffmpeg_subtitle(ncplane* parent, const ncvisual* ncv){
         continue;
       }
       struct notcurses* nc = ncplane_notcurses(parent);
-      if(nc->tcache.cellpixy <= 0 || nc->tcache.cellpixx <= 0){
+      const unsigned cellpxy = ncplane_pile_const(parent)->cellpxy;
+      const unsigned cellpxx = ncplane_pile_const(parent)->cellpxx;
+      if(cellpxy <= 0 || cellpxx <= 0){
         continue;
       }
       struct ncvisual* v = ncvisual_from_palidx(rect->data[0], rect->h,
-                                                rect->w, rect->w, 256, 1,
-                                                palette);
+                                                rect->w, rect->w,
+                                                NCPALETTESIZE, 1, palette);
       if(v == NULL){
         return NULL;
       }
-      int rows = (rect->h + nc->tcache.cellpixy - 1) / nc->tcache.cellpixy;
+      int rows = (rect->h + cellpxx - 1) / cellpxy;
       struct ncplane_options nopts = {
         .rows = rows,
-        .cols = (rect->w + nc->tcache.cellpixx - 1) / nc->tcache.cellpixx,
+        .cols = (rect->w + cellpxx - 1) / cellpxx,
         .y = ncplane_dim_y(parent) - rows - 1,
         .name = "t1st",
       };
@@ -263,15 +264,13 @@ force_rgba(ncvisual* n){
                          sframe->linesize);
   if(height < 0){
 //fprintf(stderr, "Error applying converting %d\n", inf->format);
-    av_freep(&sframe->data[0]);
-    av_freep(&sframe);
+    av_frame_free(&sframe);
     return -1;
   }
   int bpp = av_get_bits_per_pixel(av_pix_fmt_desc_get(sframe->format));
   if(bpp != 32){
 //fprintf(stderr, "Bad bits-per-pixel (wanted 32, got %d)\n", bpp);
-    av_freep(&sframe->data[0]);
-    av_freep(&sframe);
+    av_frame_free(&sframe);
     return -1;
   }
   n->rowstride = sframe->linesize[0];
@@ -290,54 +289,67 @@ force_rgba(ncvisual* n){
   return 0;
 }
 
-int ffmpeg_decode(ncvisual* n){
+// turn arbitrary input packets into RGBA frames. reads packets until it gets
+// a visual frame. a packet might contain several frames (this is typically
+// true only of audio), and a frame might be carried across several packets.
+// * avcodec_receive_frame() returns EAGAIN if it needs more packets.
+// * avcodec_send_packet() returns EAGAIN if avcodec_receive_frame() needs
+//    be called to extract further frames; in this case, the packet ought
+//    be resubmitted once the existing frames are cleared.
+static int
+ffmpeg_decode(ncvisual* n){
   if(n->details->fmtctx == NULL){ // not a file-backed ncvisual
     return -1;
   }
   bool have_frame = false;
   bool unref = false;
+  // note that there are two loops here; once we're out of the external one,
+  // we've either returned a failure, or we have a frame. averr2ncerr()
+  // translates AVERROR_EOF into a return of 1.
   do{
-    do{
-      if(n->details->packet_outstanding){
-        break;
-      }
-      if(unref){
+    if(!n->details->packet_outstanding){
+      do{
+        if(unref){
+          av_packet_unref(n->details->packet);
+        }
+        int averr;
+        if((averr = av_read_frame(n->details->fmtctx, n->details->packet)) < 0){
+          /*if(averr != AVERROR_EOF){
+            fprintf(stderr, "Error reading frame info (%s)\n", av_err2str(averr));
+          }*/
+          return averr2ncerr(averr);
+        }
+        unref = true;
+        if(n->details->packet->stream_index == n->details->sub_stream_index){
+          int result = 0, ret;
+          avsubtitle_free(&n->details->subtitle);
+          ret = avcodec_decode_subtitle2(n->details->subtcodecctx, &n->details->subtitle, &result, n->details->packet);
+          if(ret >= 0 && result){
+            // FIXME?
+          }
+        }
+      }while(n->details->packet->stream_index != n->details->stream_index);
+      n->details->packet_outstanding = true;
+      int averr = avcodec_send_packet(n->details->codecctx, n->details->packet);
+      if(averr < 0){
+        n->details->packet_outstanding = false;
         av_packet_unref(n->details->packet);
-      }
-      int averr;
-      if((averr = av_read_frame(n->details->fmtctx, n->details->packet)) < 0){
-        /*if(averr != AVERROR_EOF){
-          fprintf(stderr, "Error reading frame info (%s)\n", av_err2str(averr));
-        }*/
+  //fprintf(stderr, "Error processing AVPacket\n");
         return averr2ncerr(averr);
       }
-      unref = true;
-      if(n->details->packet->stream_index == n->details->sub_stream_index){
-        int result = 0, ret;
-        avsubtitle_free(&n->details->subtitle);
-        ret = avcodec_decode_subtitle2(n->details->subtcodecctx, &n->details->subtitle, &result, n->details->packet);
-        if(ret >= 0 && result){
-          // FIXME?
-        }
-      }
-    }while(n->details->packet->stream_index != n->details->stream_index);
-    ++n->details->packet_outstanding;
-    int averr = avcodec_send_packet(n->details->codecctx, n->details->packet);
-    if(averr < 0){
-//fprintf(stderr, "Error processing AVPacket\n");
-      return averr2ncerr(averr);
     }
-    --n->details->packet_outstanding;
-    av_packet_unref(n->details->packet);
-    averr = avcodec_receive_frame(n->details->codecctx, n->details->frame);
+    int averr = avcodec_receive_frame(n->details->codecctx, n->details->frame);
     if(averr >= 0){
       have_frame = true;
-    }else if(averr == AVERROR(EAGAIN) || averr == AVERROR_EOF){
-      have_frame = false;
     }else if(averr < 0){
-//fprintf(stderr, "Error decoding AVPacket\n");
-      return averr2ncerr(averr);
+      av_packet_unref(n->details->packet);
+      have_frame = false;
+      n->details->packet_outstanding = false;
+      if(averr != AVERROR(EAGAIN)){
+        return averr2ncerr(averr);
+      }
     }
+//fprintf(stderr, "Error decoding AVPacket\n");
   }while(!have_frame);
 //print_frame_summary(n->details->codecctx, n->details->frame);
   const AVFrame* f = n->details->frame;
@@ -350,7 +362,8 @@ int ffmpeg_decode(ncvisual* n){
   return 0;
 }
 
-ncvisual_details* ffmpeg_details_init(void){
+static ncvisual_details*
+ffmpeg_details_init(void){
   ncvisual_details* deets = malloc(sizeof(*deets));
   if(deets){
     memset(deets, 0, sizeof(*deets));
@@ -364,7 +377,8 @@ ncvisual_details* ffmpeg_details_init(void){
   return deets;
 }
 
-ncvisual* ffmpeg_create(){
+static ncvisual*
+ffmpeg_create(){
   ncvisual* nc = malloc(sizeof(*nc));
   if(nc){
     memset(nc, 0, sizeof(*nc));
@@ -376,8 +390,8 @@ ncvisual* ffmpeg_create(){
   return nc;
 }
 
-ncvisual* ffmpeg_from_file(const char* filename){
-  AVStream* st;
+static ncvisual*
+ffmpeg_from_file(const char* filename){
   ncvisual* ncv = ffmpeg_create();
   if(ncv == NULL){
     // fprintf(stderr, "Couldn't create %s (%s)\n", filename, strerror(errno));
@@ -433,7 +447,7 @@ ncvisual* ffmpeg_from_file(const char* filename){
     //fprintf(stderr, "Couldn't find decoder for %s\n", filename);
     goto err;
   }
-  st = ncv->details->fmtctx->streams[ncv->details->stream_index];
+  AVStream* st = ncv->details->fmtctx->streams[ncv->details->stream_index];
   if((ncv->details->codecctx = avcodec_alloc_context3(ncv->details->codec)) == NULL){
     //fprintf(stderr, "Couldn't allocate decoder for %s\n", filename);
     goto err;
@@ -445,14 +459,6 @@ ncvisual* ffmpeg_from_file(const char* filename){
     //fprintf(stderr, "Couldn't open codec for %s (%s)\n", filename, av_err2str(*averr));
     goto err;
   }
-  /*if((ncv->cparams = avcodec_parameters_alloc()) == NULL){
-    //fprintf(stderr, "Couldn't allocate codec params for %s\n", filename);
-    goto err;
-  }
-  if((*averr = avcodec_parameters_from_context(ncv->cparams, ncv->details->codecctx)) < 0){
-    //fprintf(stderr, "Couldn't get codec params for %s (%s)\n", filename, av_err2str(*averr));
-    goto err;
-  }*/
 //fprintf(stderr, "FRAME FRAME: %p\n", ncv->details->frame);
   // frame is set up in prep_details(), so that format can be set there, as
   // is necessary when it is prepared from inputs other than files.
@@ -470,9 +476,10 @@ err:
 // frames carry a presentation time relative to the beginning, so we get an
 // initial timestamp, and check each frame against the elapsed time to sync
 // up playback.
-int ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
-                  ncstreamcb streamer, const struct ncvisual_options* vopts,
-                  void* curry){
+static int
+ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
+              ncstreamcb streamer, const struct ncvisual_options* vopts,
+              void* curry){
   int frame = 1;
   struct timespec begin; // time we started
   clock_gettime(CLOCK_MONOTONIC, &begin);
@@ -498,9 +505,11 @@ int ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
       ncplane_erase(activevopts.n); // new frame could be partially transparent
     }
     // decay the blitter explicitly, so that the callback knows the blitter it
-    // was actually rendered with
-    ncvisual_blitter_geom(nc, ncv, &activevopts, NULL, NULL, NULL, NULL,
-                          &activevopts.blitter);
+    // was actually rendered with. basically just need rgba_blitter(), but
+    // that's not exported.
+    ncvgeom geom;
+    ncvisual_geom(nc, ncv, &activevopts, &geom);
+    activevopts.blitter = geom.blitter;
     if((newn = ncvisual_blit(nc, ncv, &activevopts)) == NULL){
       if(activevopts.n != vopts->n){
         ncplane_destroy(activevopts.n);
@@ -539,7 +548,8 @@ int ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
   return ncerr;
 }
 
-int ffmpeg_decode_loop(ncvisual* ncv){
+static int
+ffmpeg_decode_loop(ncvisual* ncv){
   int r = ffmpeg_decode(ncv);
   if(r == 1){
     if(av_seek_frame(ncv->details->fmtctx, ncv->details->stream_index, 0, AVSEEK_FLAG_FRAME) < 0){
@@ -605,7 +615,8 @@ ffmpeg_resize_internal(const ncvisual* ncv, int rows, int* stride, int cols,
 }
 
 // resize frame, converting to RGBA (if necessary) along the way
-int ffmpeg_resize(ncvisual* n, int rows, int cols){
+static int
+ffmpeg_resize(ncvisual* n, unsigned rows, unsigned cols){
   struct blitterargs bargs = {};
   int stride;
   void* data = ffmpeg_resize_internal(n, rows, &stride, cols, &bargs);
@@ -629,18 +640,18 @@ int ffmpeg_resize(ncvisual* n, int rows, int cols){
 }
 
 // rows/cols: scaled output geometry (pixels)
-int ffmpeg_blit(ncvisual* ncv, int rows, int cols, ncplane* n,
-                const struct blitset* bset, const blitterargs* bargs){
+static int
+ffmpeg_blit(ncvisual* ncv, unsigned rows, unsigned cols, ncplane* n,
+            const struct blitset* bset, const blitterargs* bargs){
   void* data;
   int stride = 0;
   data = ffmpeg_resize_internal(ncv, rows, &stride, cols, bargs);
   if(data == NULL){
     return -1;
   }
-//fprintf(stderr, "WHN NCV: %d/%d bargslen: %d/%d targ: %d/%d\n", inframe->width, inframe->height, bargs->leny, bargs->lenx, rows, cols);
+//fprintf(stderr, "WHN NCV: bargslen: %d/%d targ: %d/%d\n", bargs->leny, bargs->lenx, rows, cols);
   int ret = 0;
   if(rgba_blit_dispatch(n, bset, stride, data, rows, cols, bargs) < 0){
-//fprintf(stderr, "rgba dispatch failed!\n");
     ret = -1;
   }
   if(data != ncv->data){
@@ -649,21 +660,18 @@ int ffmpeg_blit(ncvisual* ncv, int rows, int cols, ncplane* n,
   return ret;
 }
 
-void ffmpeg_details_seed(ncvisual* ncv){
-  ncv->details->frame->data[0] = NULL;
-  ncv->details->frame->data[1] = NULL;
-  ncv->details->frame->data[2] = NULL;
-  ncv->details->frame->data[3] = NULL;
+static void
+ffmpeg_details_seed(ncvisual* ncv){
+  av_frame_unref(ncv->details->frame);
+  memset(ncv->details->frame, 0, sizeof(*ncv->details->frame));
   ncv->details->frame->linesize[0] = ncv->rowstride;
-  ncv->details->frame->linesize[1] = 0;
-  ncv->details->frame->linesize[2] = 0;
-  ncv->details->frame->linesize[3] = 0;
   ncv->details->frame->width = ncv->pixx;
   ncv->details->frame->height = ncv->pixy;
   ncv->details->frame->format = AV_PIX_FMT_RGBA;
 }
 
-int ffmpeg_log_level(int level){
+static int
+ffmpeg_log_level(int level){
   switch(level){
     case NCLOGLEVEL_SILENT: return AV_LOG_QUIET;
     case NCLOGLEVEL_PANIC: return AV_LOG_PANIC;
@@ -680,13 +688,15 @@ int ffmpeg_log_level(int level){
   return AV_LOG_TRACE;
 }
 
-int ffmpeg_init(int logl){
+static int
+ffmpeg_init(int logl){
   av_log_set_level(ffmpeg_log_level(logl));
   // FIXME could also use av_log_set_callback() and capture the message...
   return 0;
 }
 
-void ffmpeg_printbanner(fbuf* f){
+static void
+ffmpeg_printbanner(fbuf* f){
   fbuf_printf(f, "avformat %u.%u.%u avutil %u.%u.%u swscale %u.%u.%u avcodec %u.%u.%u" NL,
               LIBAVFORMAT_VERSION_MAJOR, LIBAVFORMAT_VERSION_MINOR, LIBAVFORMAT_VERSION_MICRO,
               LIBAVUTIL_VERSION_MAJOR, LIBAVUTIL_VERSION_MINOR, LIBAVUTIL_VERSION_MICRO,
@@ -694,12 +704,12 @@ void ffmpeg_printbanner(fbuf* f){
               LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO);
 }
 
-void ffmpeg_details_destroy(ncvisual_details* deets){
+static void
+ffmpeg_details_destroy(ncvisual_details* deets){
   avcodec_close(deets->codecctx);
   avcodec_free_context(&deets->subtcodecctx);
   avcodec_free_context(&deets->codecctx);
   av_frame_free(&deets->frame);
-  //avcodec_parameters_free(&ncv->cparams);
   sws_freeContext(deets->rgbactx);
   sws_freeContext(deets->swsctx);
   av_packet_free(&deets->packet);
@@ -708,7 +718,8 @@ void ffmpeg_details_destroy(ncvisual_details* deets){
   free(deets);
 }
 
-void ffmpeg_destroy(ncvisual* ncv){
+static void
+ffmpeg_destroy(ncvisual* ncv){
   if(ncv){
     ffmpeg_details_destroy(ncv->details);
     if(ncv->owndata){
@@ -718,7 +729,7 @@ void ffmpeg_destroy(ncvisual* ncv){
   }
 }
 
-const ncvisual_implementation local_visual_implementation = {
+ncvisual_implementation local_visual_implementation = {
   .visual_init = ffmpeg_init,
   .visual_printbanner = ffmpeg_printbanner,
   .visual_blit = ffmpeg_blit,
